# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import ddt
import mock
import six

from rally import consts as rally_consts
from rally import exceptions
from rally.plugins.openstack.context.manila import consts
from rally.plugins.openstack.context.manila import manila_share_networks
from tests.unit import test

MANILA_UTILS_PATH = ("rally.plugins.openstack.scenarios.manila.utils."
                     "ManilaScenario.")


class Fake(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __getitem__(self, item):
        return getattr(self, item)

    def to_dict(self):
        return self.__dict__


@ddt.ddt
class ShareNetworksTestCase(test.TestCase):
    TENANTS_AMOUNT = 3
    USERS_PER_TENANT = 4
    SECURITY_SERVICES = [
        {"type": ss_type,
         "dns_ip": "fake_dns_ip_%s" % ss_type,
         "server": "fake_server_%s" % ss_type,
         "domain": "fake_domain_%s" % ss_type,
         "user": "fake_user_%s" % ss_type,
         "password": "fake_password_%s" % ss_type,
         "name": "fake_optional_name_%s" % ss_type}
        for ss_type in ("ldap", "kerberos", "active_directory")
    ]

    def _get_context(self, use_security_services=False, networks_per_tenant=2,
                     neutron_network_provider=True):
        tenants = {}
        for t_id in range(self.TENANTS_AMOUNT):
            tenants[six.text_type(t_id)] = {"name": six.text_type(t_id)}
            tenants[six.text_type(t_id)]["networks"] = []
            for i in range(networks_per_tenant):
                network = {"id": "fake_net_id_%s" % i}
                if neutron_network_provider:
                    network["subnets"] = ["fake_subnet_id_of_net_%s" % i]
                else:
                    network["cidr"] = "101.0.5.0/24"
                tenants[six.text_type(t_id)]["networks"].append(network)
        users = []
        for t_id in tenants.keys():
            for i in range(self.USERS_PER_TENANT):
                users.append(
                    {"id": i, "tenant_id": t_id, "credential": "fake"})
        context = {
            "config": {
                "users": {
                    "tenants": self.TENANTS_AMOUNT,
                    "users_per_tenant": self.USERS_PER_TENANT,
                    "random_user_choice": False,
                },
                consts.SHARE_NETWORKS_CONTEXT_NAME: {
                    "use_share_networks": True,
                    "share_networks": [],
                },
                consts.SECURITY_SERVICES_CONTEXT_NAME: {
                    "security_services": (
                        self.SECURITY_SERVICES
                        if use_security_services else [])
                },
                "network": {
                    "networks_per_tenant": networks_per_tenant,
                    "start_cidr": "101.0.5.0/24",
                },
            },
            "admin": {
                "credential": mock.MagicMock(),
            },
            "task": mock.MagicMock(),
            "users": users,
            "tenants": tenants,
            "user_choice_method": "random",
        }
        return context

    def setUp(self):
        super(self.__class__, self).setUp()
        self.ctxt_use_existing = {
            "task": mock.MagicMock(),
            "config": {
                "existing_users": {"foo": "bar"},
                consts.SHARE_NETWORKS_CONTEXT_NAME: {
                    "use_share_networks": True,
                    "share_networks": {
                        "tenant_1_id": ["sn_1_id", "sn_2_name"],
                        "tenant_2_name": ["sn_3_id", "sn_4_name", "sn_5_id"],
                    },
                },
            },
            "tenants": {
                "tenant_1_id": {"id": "tenant_1_id", "name": "tenant_1_name"},
                "tenant_2_id": {"id": "tenant_2_id", "name": "tenant_2_name"},
            },
            "users": [
                {"tenant_id": "tenant_1_id", "credential": {"c1": "foo"}},
                {"tenant_id": "tenant_2_id", "credential": {"c2": "bar"}},
            ],
        }
        self.existing_sns = [
            Fake(id="sn_%s_id" % i, name="sn_%s_name" % i) for i in range(1, 6)
        ]

    def test_init(self):
        context = {
            "task": mock.MagicMock(),
            "config": {
                consts.SHARE_NETWORKS_CONTEXT_NAME: {"foo": "bar"},
                "not_manila": {"not_manila_key": "not_manila_value"},
            },
        }

        inst = manila_share_networks.ShareNetworks(context)

        self.assertEqual(
            {"foo": "bar", "share_networks": {}, "use_share_networks": False},
            inst.config)
        self.assertIn(
            rally_consts.JSON_SCHEMA, inst.CONFIG_SCHEMA.get("$schema"))
        self.assertFalse(inst.CONFIG_SCHEMA.get("additionalProperties"))
        self.assertEqual("object", inst.CONFIG_SCHEMA.get("type"))
        props = inst.CONFIG_SCHEMA.get("properties", {})
        self.assertEqual({"type": "object"}, props.get("share_networks"))
        self.assertEqual({"type": "boolean"}, props.get("use_share_networks"))
        self.assertEqual(450, inst.get_order())
        self.assertEqual(
            consts.SHARE_NETWORKS_CONTEXT_NAME,
            inst.get_name())

    def test_setup_share_networks_disabled(self):
        ctxt = {
            "task": mock.MagicMock(),
            "config": {
                consts.SHARE_NETWORKS_CONTEXT_NAME: {
                    "use_share_networks": False,
                },
            },
            consts.SHARE_NETWORKS_CONTEXT_NAME: {
                "delete_share_networks": False,
            },
        }
        inst = manila_share_networks.ShareNetworks(ctxt)

        expected_ctxt = copy.deepcopy(inst.context)

        inst.setup()

        self.assertEqual(expected_ctxt, inst.context)

    @mock.patch("rally.osclients.Clients")
    @mock.patch(MANILA_UTILS_PATH + "_list_share_networks")
    def test_setup_use_existing_share_networks(
            self, mock_manila_scenario__list_share_networks, mock_clients):
        existing_sns = self.existing_sns
        expected_ctxt = copy.deepcopy(self.ctxt_use_existing)
        inst = manila_share_networks.ShareNetworks(self.ctxt_use_existing)
        mock_manila_scenario__list_share_networks.return_value = (
            self.existing_sns)
        expected_ctxt.update({
            "delete_share_networks": False,
            "tenants": {
                "tenant_1_id": {
                    "id": "tenant_1_id",
                    "name": "tenant_1_name",
                    consts.SHARE_NETWORKS_CONTEXT_NAME: {
                        "share_networks": [
                            sn.to_dict() for sn in existing_sns[0:2]],
                    },
                },
                "tenant_2_id": {
                    "id": "tenant_2_id",
                    "name": "tenant_2_name",
                    consts.SHARE_NETWORKS_CONTEXT_NAME: {
                        "share_networks": [
                            sn.to_dict() for sn in existing_sns[2:5]],
                    },
                },
            }
        })

        inst.setup()

        self.assertEqual(expected_ctxt["task"], inst.context.get("task"))
        self.assertEqual(expected_ctxt["config"], inst.context.get("config"))
        self.assertEqual(expected_ctxt["users"], inst.context.get("users"))
        self.assertEqual(
            False,
            inst.context.get(consts.SHARE_NETWORKS_CONTEXT_NAME, {}).get(
                "delete_share_networks"))
        self.assertEqual(expected_ctxt["tenants"], inst.context.get("tenants"))

    def test_setup_use_existing_share_networks_tenant_not_found(self):
        ctxt = copy.deepcopy(self.ctxt_use_existing)
        ctxt.update({"tenants": {}})
        inst = manila_share_networks.ShareNetworks(ctxt)

        self.assertRaises(exceptions.ContextSetupFailure, inst.setup)

    @mock.patch("rally.osclients.Clients")
    @mock.patch(MANILA_UTILS_PATH + "_list_share_networks")
    def test_setup_use_existing_share_networks_sn_not_found(
            self, mock_manila_scenario__list_share_networks, mock_clients):
        ctxt = copy.deepcopy(self.ctxt_use_existing)
        ctxt["config"][consts.SHARE_NETWORKS_CONTEXT_NAME][
            "share_networks"] = {"tenant_1_id": ["foo"]}
        inst = manila_share_networks.ShareNetworks(ctxt)
        mock_manila_scenario__list_share_networks.return_value = (
            self.existing_sns)

        self.assertRaises(exceptions.ContextSetupFailure, inst.setup)

    def test_setup_use_existing_share_networks_with_empty_list(self):
        ctxt = copy.deepcopy(self.ctxt_use_existing)
        ctxt["config"][consts.SHARE_NETWORKS_CONTEXT_NAME][
            "share_networks"] = {}
        inst = manila_share_networks.ShareNetworks(ctxt)

        self.assertRaises(exceptions.ContextSetupFailure, inst.setup)

    @ddt.data(True, False)
    @mock.patch("rally.osclients.Clients")
    @mock.patch(MANILA_UTILS_PATH + "_create_share_network")
    @mock.patch(MANILA_UTILS_PATH + "_add_security_service_to_share_network")
    def test_setup_autocreate_share_networks_with_security_services(
            self,
            neutron,
            mock_manila_scenario__add_security_service_to_share_network,
            mock_manila_scenario__create_share_network,
            mock_clients):
        networks_per_tenant = 2
        ctxt = self._get_context(
            networks_per_tenant=networks_per_tenant,
            neutron_network_provider=neutron,
            use_security_services=True,
        )
        inst = manila_share_networks.ShareNetworks(ctxt)
        for tenant_id in list(ctxt["tenants"].keys()):
            inst.context["tenants"][tenant_id][
                consts.SECURITY_SERVICES_CONTEXT_NAME] = {
                    "security_services": [
                        Fake(id="fake_id").to_dict() for i in (1, 2, 3)
                    ]
            }

        inst.setup()

        self.assertEqual(ctxt["task"], inst.context.get("task"))
        self.assertEqual(ctxt["config"], inst.context.get("config"))
        self.assertEqual(ctxt["users"], inst.context.get("users"))
        self.assertEqual(ctxt["tenants"], inst.context.get("tenants"))
        mock_add_security_service_to_share_network = (
            mock_manila_scenario__add_security_service_to_share_network)
        mock_add_security_service_to_share_network.assert_has_calls([
            mock.call(mock.ANY, mock.ANY)
            for i in range(
                self.TENANTS_AMOUNT *
                networks_per_tenant *
                len(self.SECURITY_SERVICES))])
        if neutron:
            sn_args = {
                "neutron_net_id": mock.ANY,
                "neutron_subnet_id": mock.ANY,
            }
        else:
            sn_args = {"nova_net_id": mock.ANY}
        expected_calls = [
            mock.call(**sn_args),
            mock.call().to_dict(),
            mock.ANY,
            mock.ANY,
            mock.ANY,
        ]
        mock_manila_scenario__create_share_network.assert_has_calls(
            expected_calls * (self.TENANTS_AMOUNT * networks_per_tenant))
        mock_clients.assert_has_calls([
            mock.call("fake", {}) for i in range(self.TENANTS_AMOUNT)])

    @ddt.data(True, False)
    @mock.patch("rally.osclients.Clients")
    @mock.patch(MANILA_UTILS_PATH + "_create_share_network")
    @mock.patch(MANILA_UTILS_PATH + "_add_security_service_to_share_network")
    def test_setup_autocreate_share_networks_wo_security_services(
            self,
            neutron,
            mock_manila_scenario__add_security_service_to_share_network,
            mock_manila_scenario__create_share_network,
            mock_clients):
        networks_per_tenant = 2
        ctxt = self._get_context(
            networks_per_tenant=networks_per_tenant,
            neutron_network_provider=neutron,
        )
        inst = manila_share_networks.ShareNetworks(ctxt)

        inst.setup()

        self.assertEqual(ctxt["task"], inst.context.get("task"))
        self.assertEqual(ctxt["config"], inst.context.get("config"))
        self.assertEqual(ctxt["users"], inst.context.get("users"))
        self.assertEqual(ctxt["tenants"], inst.context.get("tenants"))
        self.assertFalse(
            mock_manila_scenario__add_security_service_to_share_network.called)
        if neutron:
            sn_args = {
                "neutron_net_id": mock.ANY,
                "neutron_subnet_id": mock.ANY,
            }
        else:
            sn_args = {"nova_net_id": mock.ANY}
        expected_calls = [mock.call(**sn_args), mock.call().to_dict()]
        mock_manila_scenario__create_share_network.assert_has_calls(
            expected_calls * (self.TENANTS_AMOUNT * networks_per_tenant))
        mock_clients.assert_has_calls([
            mock.call("fake", {}) for i in range(self.TENANTS_AMOUNT)])

    @mock.patch("rally.osclients.Clients")
    @mock.patch(MANILA_UTILS_PATH + "_create_share_network")
    @mock.patch(MANILA_UTILS_PATH + "_add_security_service_to_share_network")
    def test_setup_autocreate_share_networks_wo_networks(
            self,
            mock_manila_scenario__add_security_service_to_share_network,
            mock_manila_scenario__create_share_network,
            mock_clients):
        ctxt = self._get_context(networks_per_tenant=0)
        inst = manila_share_networks.ShareNetworks(ctxt)

        inst.setup()

        self.assertEqual(ctxt["task"], inst.context.get("task"))
        self.assertEqual(ctxt["config"], inst.context.get("config"))
        self.assertEqual(ctxt["users"], inst.context.get("users"))
        self.assertEqual(ctxt["tenants"], inst.context.get("tenants"))
        self.assertFalse(
            mock_manila_scenario__add_security_service_to_share_network.called)
        expected_calls = [mock.call(), mock.call().to_dict()]
        mock_manila_scenario__create_share_network.assert_has_calls(
            expected_calls * self.TENANTS_AMOUNT)
        mock_clients.assert_has_calls([
            mock.call("fake", {}) for i in range(self.TENANTS_AMOUNT)])

    @mock.patch("rally.osclients.Clients")
    @mock.patch(MANILA_UTILS_PATH + "_delete_share_network")
    @mock.patch(MANILA_UTILS_PATH + "_list_share_servers")
    @mock.patch(MANILA_UTILS_PATH + "_list_share_networks")
    def test_cleanup_used_existing_share_networks(
            self,
            mock_manila_scenario__list_share_networks,
            mock_manila_scenario__list_share_servers,
            mock_manila_scenario__delete_share_network,
            mock_clients):
        inst = manila_share_networks.ShareNetworks(self.ctxt_use_existing)
        mock_manila_scenario__list_share_networks.return_value = (
            self.existing_sns)
        inst.setup()

        inst.cleanup()

        self.assertFalse(mock_manila_scenario__list_share_servers.called)
        self.assertFalse(mock_manila_scenario__delete_share_network.called)
        self.assertEqual(2, mock_clients.call_count)
        for user in self.ctxt_use_existing["users"]:
            self.assertIn(mock.call(user["credential"], {}),
                          mock_clients.mock_calls)

    @ddt.data(True, False)
    @mock.patch("rally.task.utils.wait_for_status")
    @mock.patch("rally.osclients.Clients")
    @mock.patch(MANILA_UTILS_PATH + "_delete_share_network")
    @mock.patch(MANILA_UTILS_PATH + "_create_share_network")
    @mock.patch(MANILA_UTILS_PATH + "_add_security_service_to_share_network")
    @mock.patch(MANILA_UTILS_PATH + "_list_share_servers")
    def test_cleanup_autocreated_share_networks(
            self, use_security_services,
            mock_manila_scenario__list_share_servers,
            mock_manila_scenario__add_security_service_to_share_network,
            mock_manila_scenario__create_share_network,
            mock_manila_scenario__delete_share_network,
            mock_clients,
            mock_wait_for_status):
        fake_share_servers = ["fake_share_server"]
        mock_manila_scenario__list_share_servers.return_value = (
            fake_share_servers)
        networks_per_tenant = 2
        ctxt = self._get_context(
            networks_per_tenant=networks_per_tenant,
            use_security_services=use_security_services,
        )
        inst = manila_share_networks.ShareNetworks(ctxt)
        for tenant_id in list(ctxt["tenants"].keys()):
            inst.context["tenants"][tenant_id][
                consts.SECURITY_SERVICES_CONTEXT_NAME] = {
                    "security_services": [
                        Fake(id="fake_id").to_dict() for i in (1, 2, 3)
                    ]
            }
        inst.setup()

        mock_clients.assert_has_calls([
            mock.call("fake", {}) for i in range(self.TENANTS_AMOUNT)])

        inst.cleanup()

        self.assertEqual(self.TENANTS_AMOUNT * 4, mock_clients.call_count)
        self.assertEqual(
            self.TENANTS_AMOUNT * networks_per_tenant,
            mock_manila_scenario__list_share_servers.call_count)
        mock_manila_scenario__list_share_servers.assert_has_calls(
            [mock.call(search_opts=mock.ANY)])
        self.assertEqual(
            self.TENANTS_AMOUNT * networks_per_tenant,
            mock_manila_scenario__delete_share_network.call_count)
        self.assertEqual(
            self.TENANTS_AMOUNT * networks_per_tenant,
            mock_wait_for_status.call_count)
        mock_wait_for_status.assert_has_calls([
            mock.call(
                fake_share_servers[0],
                ready_statuses=["deleted"],
                check_deletion=True,
                update_resource=mock.ANY,
                timeout=180,
                check_interval=2),
        ])



# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from rally.plugins.openstack.context.cinder import volume_types
from tests.unit import test

CTX = "rally.plugins.openstack.context"


class VolumeTypeGeneratorTestCase(test.ContextTestCase):
    def setUp(self):
        super(VolumeTypeGeneratorTestCase, self).setUp()
        self.context.update({"admin": {"credential": "admin_creds"}})

    def test_setup(self):
        self.context.update({"config": {"volume_types": ["foo", "bar"]}})
        create = self.clients("cinder",
                              admin=True).volume_types.create
        create.side_effect = (mock.Mock(id="foo-id"),
                              mock.Mock(id="bar-id"))

        vtype_ctx = volume_types.VolumeTypeGenerator(self.context)
        vtype_ctx.setup()

        create.assert_has_calls(
            [mock.call("foo"), mock.call("bar")])
        self.assertEqual(self.context["volume_types"],
                         [{"id": "foo-id", "name": "foo"},
                          {"id": "bar-id", "name": "bar"}])

    def test_cleanup(self):
        self.context.update({
            "config": {"volume_types": ["foo", "bar"]},
            "volume_types": [
                {"id": "foo_id", "name": "foo"},
                {"id": "bar_id", "name": "bar"}],
            "api_versions": {
                "cinder": {"version": 2, "service_type": "volumev2"}}})

        vtype_ctx = volume_types.VolumeTypeGenerator(self.context)
        vtype_ctx.cleanup()

        delete = self.clients("cinder",
                              admin=True).volume_types.delete
        delete.assert_has_calls(
            [mock.call("foo_id"), mock.call("bar_id")])



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import os
import subprocess
import sys


PYTEST_REPORT = os.environ.get("PYTEST_REPORT",
                               ".test_results/pytest_results.html")
TESTR_REPORT = "testr_results.html"
PYTEST_ARGUMENTS = ("py.test"  # base command
                    " --html=%(html_report)s"  # html report
                    " --durations=10"  # get a list of the slowest 10 tests
                    " %(path)s"
                    )


def error(msg):
    print(msg)
    exit(1)


def main(args):
    parser = argparse.ArgumentParser(args[0])
    parser.add_argument("discovery_path", metavar="<path>", type=str,
                        help="Path to location of all tests.")
    parser.add_argument("--posargs", metavar="<str>", type=str, default="",
                        help="TOX posargs. Currently supported only string to "
                             "partial test or tests group to launch.")
    parser.add_argument("--timeout", metavar="<seconds>", type=int, default=60,
                        help="Timeout for individual test execution. "
                             "Defaults to 60")
    args = parser.parse_args(args[1:])

    # We allow only one parameter - path to partial test or tests group
    path = args.posargs
    if len(path.split(" ")) > 1:
        error("Wrong value of posargs. It should include only path to single "
              "test or tests group to launch.")
    # NOTE(andreykurilin): Previously, next format was supported:
    #   tests.unit.test_osclients.SomeTestCase.some_method
    # It is more simple and pythonic than native pytest-way:
    #   tests/unit/test_osclients.py::SomeTestCase::some_method
    # Let's return this support
    if path:
        if "/" not in path:
            path = path.split(".")
            module = ""
            for i in range(0, len(path)):
                part = os.path.join(module, path[i])
                if os.path.exists(part):
                    module = part
                    continue
                if os.path.exists("%s.py" % part):
                    if i != (len(path) - 1):
                        module = "%s.py::%s" % (part, "::".join(path[i + 1:]))
                    else:
                        module = "%s.py" % part
                    break

                error("Non-existing path to single test or tests group to "
                      "launch. %s %s" % (module, part))
            path = module

        path = os.path.abspath(os.path.expanduser(path))
        if not path.startswith(os.path.abspath(args.discovery_path)):
            # Prevent to launch functional tests from unit tests launcher.
            error("Wrong path to single test or tests group to launch. It "
                  "should be in %s." % args.discovery_path)
    else:
        path = args.discovery_path

    print("Test(s) to launch (pytest format): %s" % path)

    # NOTE(andreykurilin): we cannot publish pytest reports at gates, but we
    #   can mask them as testr reports. It looks like a dirty hack and I
    #   prefer to avoid it, but I see no other solutions at this point.

    # apply dirty hack only in gates.
    if os.environ.get("ZUUL_PROJECT"):
        pytest_report = TESTR_REPORT
    else:
        pytest_report = PYTEST_REPORT

    args = PYTEST_ARGUMENTS % {"html_report": pytest_report,
                               "path": path,
                               "timeout": args.timeout}
    try:
        subprocess.check_call(args.split(" "),
                              stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError:
        # NOTE(andreykurilin): it is ok, since tests can fail.
        exit_code = 1
    else:
        exit_code = 0

    if os.path.exists(pytest_report) and os.environ.get("ZUUL_PROJECT"):
        subprocess.check_call(["gzip", "-9", "-f", pytest_report],
                              stderr=subprocess.STDOUT)

    if exit_code == 1:
        error("")

if __name__ == "__main__":
    sys.exit(main(sys.argv))



# Copyright 2016: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import jsonschema

from rally.plugins.common.sla import max_average_duration_per_atomic as madpa
from tests.unit import test


@ddt.ddt
class MaxAverageDurationPerAtomicTestCase(test.TestCase):
    def test_config_schema(self):
        properties = {
            "max_avg_duration_per_atomic": {"neutron.list_ports": "elf",
                                            "neutron.create_port": 1.0}
        }
        self.assertRaises(
            jsonschema.ValidationError,
            madpa.MaxAverageDurationPerAtomic.validate,
            properties)
        properties["max_avg_duration_per_atomic"]["neutron.list_ports"] = 1.0
        madpa.MaxAverageDurationPerAtomic.validate(properties)

    def test_result(self):
        cls = madpa.MaxAverageDurationPerAtomic
        sla1 = cls({"a1": 42, "a2": 42})
        sla2 = cls({"a1": 42, "a2": 2})
        for sla in [sla1, sla2]:
            sla.add_iteration({"atomic_actions": {"a1": 3.14, "a2": 7.77}})
            sla.add_iteration({"atomic_actions": {"a1": 8.14, "a2": 9.77}})
        self.assertTrue(sla1.result()["success"])
        self.assertFalse(sla2.result()["success"])
        self.assertEqual("Passed", sla1.status())
        self.assertEqual("Failed", sla2.status())

    def test_result_no_iterations(self):
        sla = madpa.MaxAverageDurationPerAtomic({"a1": 8.14, "a2": 9.77})
        self.assertTrue(sla.result()["success"])

    def test_add_iteration(self):
        sla = madpa.MaxAverageDurationPerAtomic({"a1": 5, "a2": 10})
        add = sla.add_iteration
        self.assertTrue(add({"atomic_actions": {"a1": 2.5, "a2": 5.0}}))
        self.assertTrue(add({"atomic_actions": {"a1": 5.0, "a2": 10.0}}))
        # the following pushes a2 over the limit
        self.assertFalse(add({"atomic_actions": {"a1": 5.0, "a2": 20.0}}))
        # bring a2 back
        self.assertTrue(add({"atomic_actions": {"a1": 5.0, "a2": 2.0}}))
        # push a1 over
        self.assertFalse(add({"atomic_actions": {"a1": 10.0, "a2": 2.0}}))
        # bring it back
        self.assertTrue(add({"atomic_actions": {"a1": 1.0, "a2": 2.0}}))

    @ddt.data([[1.0, 2.0, 1.5, 4.3],
               [2.1, 3.4, 1.2, 6.3, 7.2, 7.0, 1.],
               [1.1, 1.1, 2.2, 2.2, 3.3, 4.3]])
    def test_merge(self, durations):
        init = {"a1": 8.14, "a2": 9.77}
        single_sla = madpa.MaxAverageDurationPerAtomic(init)

        for dd in durations:
            for d in dd:
                single_sla.add_iteration(
                    {"atomic_actions": {"a1": d, "a2": d * 2}})

        slas = [madpa.MaxAverageDurationPerAtomic(init) for _ in durations]

        for idx, sla in enumerate(slas):
            for d in durations[idx]:
                sla.add_iteration({"atomic_actions": {"a1": d, "a2": d * 2}})

        merged_sla = slas[0]
        for sla in slas[1:]:
            merged_sla.merge(sla)

        self.assertEqual(single_sla.success, merged_sla.success)
        self.assertEqual(single_sla.avg_by_action, merged_sla.avg_by_action)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from rally.plugins.openstack.scenarios.cinder import volume_types
from tests.unit import test


class fake_type(object):
    name = "fake"


class CinderVolumeTypesTestCase(test.ScenarioTestCase):

    def _get_context(self):
        context = test.get_test_context()
        context.update({
            "volume_types": [{"id": "fake_id",
                              "name": "fake_name"}]})
        return context

    def test_create_and_delete_volume_type(self):
        scenario = volume_types.CreateAndDeleteVolumeType(self.context)
        scenario._create_volume_type = mock.Mock()
        scenario._delete_volume_type = mock.Mock()
        scenario.run(fakeargs="fakeargs")
        scenario._create_volume_type.assert_called_once_with(
            fakeargs="fakeargs")
        scenario._delete_volume_type.assert_called_once_with(
            scenario._create_volume_type.return_value)

    def test_create_and_delete_encryption_type(self):
        scenario = volume_types.CreateAndDeleteEncryptionType(
            self._get_context())
        scenario._create_encryption_type = mock.Mock()
        scenario._delete_encryption_type = mock.Mock()
        scenario.run(create_specs="fakecreatespecs")
        scenario._create_encryption_type.assert_called_once_with(
            "fake_id", "fakecreatespecs")
        scenario._delete_encryption_type.assert_called_once_with(
            "fake_id")

    def test_create_volume_type_and_encryption_type(self):
        scenario = volume_types.CreateVolumeTypeAndEncryptionType(self.context)
        scenario._create_volume_type = mock.Mock()
        scenario._create_encryption_type = mock.Mock()
        scenario.run(specs="fakespecs", fakeargs="fakeargs")
        scenario._create_volume_type.assert_called_once_with(
            fakeargs="fakeargs")
        scenario._create_encryption_type.assert_called_once_with(
            scenario._create_volume_type.return_value, "fakespecs")

    def test_create_and_list_encryption_type(self):
        scenario = volume_types.CreateAndListEncryptionType(self.context)
        scenario._create_volume_type = mock.Mock()
        scenario._create_encryption_type = mock.Mock()
        scenario._list_encryption_type = mock.Mock()
        scenario.run(specs="fakespecs", search_opts="fakeopts",
                     fakeargs="fakeargs")
        scenario._create_volume_type.assert_called_once_with(
            fakeargs="fakeargs")
        scenario._create_encryption_type.assert_called_once_with(
            scenario._create_volume_type.return_value, "fakespecs")
        scenario._list_encryption_type.assert_called_once_with(
            "fakeopts")

    def test_create_and_set_volume_type_keys(self):
        scenario = volume_types.CreateAndSetVolumeTypeKeys(self.context)

        volume_type = mock.MagicMock()
        volume_type_key = {"volume_backend_name": "LVM_iSCSI"}
        scenario._create_volume_type = mock.MagicMock()
        scenario._set_volume_type_keys = mock.MagicMock()

        scenario._create_volume_type.return_value = volume_type
        scenario.run(volume_type_key, fakeargs="fakeargs")

        scenario._create_volume_type.assert_called_once_with(
            fakeargs="fakeargs")
        scenario._set_volume_type_keys.assert_called_once_with(volume_type,
                                                               volume_type_key)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from rally import plugins
from rally.common.plugin import discover
from rally.common.plugin import info
from rally.common.plugin import plugin
from rally.deployment import engine
from rally.deployment.serverprovider import provider
from rally.task import sla
from tests.unit import test

EXCEPTIONS_DOCSTR = "missed_docstrings.txt"
EXCEPTIONS_FORMAT = "wrong_format.txt"


class DocstringsTestCase(test.TestCase):


    def setUp(self):
        super(DocstringsTestCase, self).setUp()
        plugins.load()

        self.exceptions = self._open_file(
            EXCEPTIONS_DOCSTR) + self._open_file(EXCEPTIONS_FORMAT)

    def _open_file(self, filename):
        with open("./tests/unit/doc/%s" % filename) as file:
            return (file.read().lower().split())

    def _check_docstrings(self, msg_buffer):
        for plg_cls in plugin.Plugin.get_all():
            if plg_cls.__module__.startswith("rally."):
                if plg_cls.get_name().lower() not in self.exceptions:
                    doc = info.parse_docstring(plg_cls.__doc__)
                    short_description = doc["short_description"]
                    if short_description.startswith("Test"):
                        msg_buffer.append("One-line description for %s"
                                          " should be declarative and not"
                                          " start with 'Test(s) ...'"
                                          % plg_cls.__name__)
                    if not plg_cls.get_info()["title"]:
                        msg = ("Class '{}' should have a docstring.")
                        inst_name = plg_cls.__name__
                        msg_buffer.append(msg.format(inst_name))

    def _check_described_params(self, msg_buffer):
        for plg_cls in plugin.Plugin.get_all():
            if plg_cls.get_name().lower() not in self.exceptions:
                ignored_params = ["self", "scenario_obj"]
                if hasattr(plg_cls, "run"):
                    code_block = plg_cls.run.__code__
                    params_count = code_block.co_argcount
                    params = code_block.co_varnames[:params_count]
                    param_data = plg_cls.get_info()["parameters"]
                    documented_params = [p["name"] for p in param_data]
                    for param in params:
                        if param not in ignored_params:
                            if param not in documented_params:
                                msg = ("Class: %(class)s Docstring for "
                                       "%(scenario)s should"
                                       " describe the '%(param)s' parameter"
                                       " in the :param <name>: clause."
                                       % {"class": plg_cls.__name__,
                                          "scenario": plg_cls.get_name(),
                                          "param": param})
                                msg_buffer.append(msg)

    def test_all_plugins_have_docstrings(self):

        msg_buffer = []
        self._check_docstrings(msg_buffer)
        if msg_buffer:
            self.fail("\n%s" % "\n".join(msg_buffer))

        msg_buffer = []
        self._check_described_params(msg_buffer)
        if msg_buffer:
            self.fail("\n%s" % "\n".join(msg_buffer))



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


"""List and compare most used OpenStack cloud resources."""


import argparse
import json
import subprocess
import sys

import six

from rally.cli import cliutils
from rally.common import objects
from rally.common.plugin import discover
from rally import consts
from rally import osclients


def skip_if_service(service):
    def wrapper(func):
        def inner(self):
            if service in self.clients.services().values():
                return []
            return func(self)
        return inner
    return wrapper


class ResourceManager(object):

    REQUIRED_SERVICE = None
    STR_ATTRS = ("id", "name")

    def __init__(self, clients):
        self.clients = clients

    def is_available(self):
        if self.REQUIRED_SERVICE:
            return self.REQUIRED_SERVICE in self.clients.services().values()
        return True

    @property
    def client(self):
        return getattr(self.clients, self.__class__.__name__.lower())()

    def get_resources(self):
        all_resources = []
        cls = self.__class__.__name__.lower()
        for prop in dir(self):
            if not prop.startswith("list_"):
                continue
            f = getattr(self, prop)
            resources = f() or []
            resource_name = prop[5:][:-1]

            for raw_res in resources:
                res = {"cls": cls, "resource_name": resource_name,
                       "id": {}, "props": {}}
                if not isinstance(raw_res, dict):
                    raw_res = {k: getattr(raw_res, k) for k in dir(raw_res)
                               if not k.startswith("_")
                               if not callable(getattr(raw_res, k))}
                for key, value in raw_res.items():
                    if key.startswith("_"):
                        continue
                    if key in self.STR_ATTRS:
                        res["id"][key] = value
                    else:
                        try:
                            res["props"][key] = json.dumps(value, indent=2)
                        except TypeError:
                            res["props"][key] = str(value)
                if not res["id"] and not res["props"]:
                    raise ValueError("Failed to represent resource %r" %
                                     raw_res)
                all_resources.append(res)
        return all_resources


class Keystone(ResourceManager):

    REQUIRED_SERVICE = consts.Service.KEYSTONE

    def list_users(self):
        return self.client.users.list()

    def list_tenants(self):
        if hasattr(self.client, "projects"):
            return self.client.projects.list()  # V3
        return self.client.tenants.list()  # V2

    def list_roles(self):
        return self.client.roles.list()

    def list_ec2credentials(self):
        users = self.list_users()
        ec2_list = []
        for user in users:
            ec2_list.extend(
                self.client.ec2.list(user.id))
        return ec2_list


class Magnum(ResourceManager):

    REQUIRED_SERVICE = consts.Service.MAGNUM

    def list_cluster_templates(self):
        result = []
        marker = None
        while True:
            ct_list = self.client.cluster_templates.list(marker=marker)
            if not ct_list:
                break
            result.extend(ct_list)
            marker = ct_list[-1].uuid
        return result

    def list_clusters(self):
        result = []
        marker = None
        while True:
            clusters = self.client.clusters.list(marker=marker)
            if not clusters:
                break
            result.extend(clusters)
            marker = clusters[-1].uuid
        return result


class Mistral(ResourceManager):

    REQUIRED_SERVICE = consts.Service.MISTRAL

    def list_workbooks(self):
        return self.client.workbooks.list()

    def list_workflows(self):
        return self.client.workflows.list()

    def list_executions(self):
        return self.client.executions.list()


class Nova(ResourceManager):

    REQUIRED_SERVICE = consts.Service.NOVA

    def list_flavors(self):
        return self.client.flavors.list()

    @skip_if_service(consts.Service.NEUTRON)
    def list_floating_ip_pools(self):
        return self.client.floating_ip_pools.list()

    @skip_if_service(consts.Service.NEUTRON)
    def list_floating_ips(self):
        return self.client.floating_ips.list()

    @skip_if_service(consts.Service.NEUTRON)
    def list_floating_ips_bulk(self):
        return self.client.floating_ips_bulk.list()

    def list_aggregates(self):
        return self.client.aggregates.list()

    def list_hosts(self):
        return self.client.hosts.list()

    def list_hypervisors(self):
        return self.client.hypervisors.list()

    def list_agents(self):
        return self.client.agents.list()

    def list_keypairs(self):
        return self.client.keypairs.list()

    @skip_if_service(consts.Service.NEUTRON)
    def list_networks(self):
        return self.client.networks.list()

    @skip_if_service(consts.Service.NEUTRON)
    def list_security_groups(self):
        return self.client.security_groups.list(
            search_opts={"all_tenants": True})

    def list_servers(self):
        return self.client.servers.list(
            search_opts={"all_tenants": True})

    def list_services(self):
        return self.client.services.list()

    def list_availability_zones(self):
        return self.client.availability_zones.list()


class Neutron(ResourceManager):

    REQUIRED_SERVICE = consts.Service.NEUTRON

    def has_extension(self, name):
        extensions = self.client.list_extensions().get("extensions", [])
        return any(ext.get("alias") == name for ext in extensions)

    def list_networks(self):
        return self.client.list_networks()["networks"]

    def list_subnets(self):
        return self.client.list_subnets()["subnets"]

    def list_routers(self):
        return self.client.list_routers()["routers"]

    def list_ports(self):
        return self.client.list_ports()["ports"]

    def list_floatingips(self):
        return self.client.list_floatingips()["floatingips"]

    def list_security_groups(self):
        return self.client.list_security_groups()["security_groups"]

    def list_health_monitors(self):
        if self.has_extension("lbaas"):
            return self.client.list_health_monitors()["health_monitors"]

    def list_pools(self):
        if self.has_extension("lbaas"):
            return self.client.list_pools()["pools"]

    def list_vips(self):
        if self.has_extension("lbaas"):
            return self.client.list_vips()["vips"]


class Glance(ResourceManager):

    REQUIRED_SERVICE = consts.Service.GLANCE

    def list_images(self):
        return self.client.images.list()


class Heat(ResourceManager):

    REQUIRED_SERVICE = consts.Service.HEAT

    def list_resource_types(self):
        return self.client.resource_types.list()

    def list_stacks(self):
        return self.client.stacks.list()


class Cinder(ResourceManager):

    REQUIRED_SERVICE = consts.Service.CINDER

    def list_availability_zones(self):
        return self.client.availability_zones.list()

    def list_backups(self):
        return self.client.backups.list()

    def list_volume_snapshots(self):
        return self.client.volume_snapshots.list()

    def list_volume_types(self):
        return self.client.volume_types.list()

    def list_encryption_types(self):
        return self.client.volume_encryption_types.list()

    def list_transfers(self):
        return self.client.transfers.list()

    def list_volumes(self):
        # ignore cache volumes for images
        volumes = self.client.volumes.list(search_opts={"all_tenants": True})
        return [v for v in volumes
                if not v.name.startswith("image-")]


class Senlin(ResourceManager):

    REQUIRED_SERVICE = consts.Service.SENLIN

    def list_clusters(self):
        return self.client.clusters()

    def list_profiles(self):
        return self.client.profiles()


class Watcher(ResourceManager):

    REQUIRED_SERVICE = consts.Service.WATCHER

    REPR_KEYS = ("uuid", "name")

    def list_audits(self):
        return self.client.audit.list()

    def list_audit_templates(self):
        return self.client.audit_template.list()

    def list_goals(self):
        return self.client.goal.list()

    def list_strategies(self):
        return self.client.strategy.list()

    def list_action_plans(self):
        return self.client.action_plan.list()


class CloudResources(object):
    """List and compare cloud resources.

    resources = CloudResources(auth_url=..., ...)
    saved_list = resources.list()

    # Do something with the cloud ...

    changes = resources.compare(saved_list)
    has_changed = any(changes)
    removed, added = changes
    """

    def __init__(self, **kwargs):
        self.clients = osclients.Clients(objects.Credential(**kwargs))

    def list(self):
        managers_classes = discover.itersubclasses(ResourceManager)
        resources = []
        for cls in managers_classes:
            manager = cls(self.clients)
            if manager.is_available():
                resources.extend(manager.get_resources())
        return resources

    def compare(self, with_list):
        def make_uuid(res):
            return"%s.%s:%s" % (
                res["cls"], res["resource_name"],
                ";".join(["%s=%s" % (k, v)
                          for k, v in sorted(res["id"].items())]))

        current_resources = dict((make_uuid(r), r) for r in self.list())
        saved_resources = dict((make_uuid(r), r) for r in with_list)

        removed = set(saved_resources.keys()) - set(current_resources.keys())
        removed = [saved_resources[k] for k in sorted(removed)]
        added = set(current_resources.keys()) - set(saved_resources.keys())
        added = [current_resources[k] for k in sorted(added)]

        return removed, added


def _print_tabular_resources(resources, table_label):
    def dict_formatter(d):
        return "\n".join("%s:%s" % (k, v) for k, v in d.items())

    cliutils.print_list(
        objs=[dict(r) for r in resources],
        fields=("cls", "resource_name", "id", "fields"),
        field_labels=("service", "resource type", "id", "fields"),
        table_label=table_label,
        formatters={"id": lambda d: dict_formatter(d["id"]),
                    "fields": lambda d: dict_formatter(d["props"])}
    )
    print("")


def main():

    parser = argparse.ArgumentParser(
        description=("Save list of OpenStack cloud resources or compare "
                     "with previously saved list."))
    parser.add_argument("--credentials",
                        type=argparse.FileType("r"),
                        metavar="<path/to/credentials.json>",
                        help="cloud credentials in JSON format")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("--dump-list",
                       type=argparse.FileType("w"),
                       metavar="<path/to/output/list.json>",
                       help="dump resources to given file in JSON format")
    group.add_argument("--compare-with-list",
                       type=argparse.FileType("r"),
                       metavar="<path/to/existent/list.json>",
                       help=("compare current resources with a list from "
                             "given JSON file"))
    args = parser.parse_args()

    if args.credentials:
        config = json.load(args.credentials)
    else:
        out = subprocess.check_output(["rally", "deployment",
                                       "config"])
        config = json.loads(out if six.PY2 else out.decode("utf-8"))
        config.update(config.pop("admin"))
        del config["type"]
        if "users" in config:
            del config["users"]

    resources = CloudResources(**config)

    if args.dump_list:
        resources_list = resources.list()
        json.dump(resources_list, args.dump_list)
    elif args.compare_with_list:
        given_list = json.load(args.compare_with_list)
        changes = resources.compare(with_list=given_list)
        removed, added = changes

        # filter out expected additions
        expected = []
        for resource in added:
            if ((resource["cls"] == "keystone" and
                 resource["resource_name"] == "role" and
                 resource["id"].get("name") == "_member_") or
                (resource["cls"] == "nova" and
                 resource["resource_name"] == "security_group" and
                 resource["id"].get("name") == "default")):
                expected.append(resource)
        for resource in expected:
            added.remove(resource)

        if removed:
            _print_tabular_resources(removed, "Removed resources")

        if added:
            _print_tabular_resources(added, "Added resources (unexpected)")

        if expected:
            _print_tabular_resources(expected, "Added resources (expected)")

        if any(changes):
            return 0  # `1' will fail gate job
    return 0


if __name__ == "__main__":
    sys.exit(main())



# Copyright 2015: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from rally import exceptions
from rally.plugins.openstack.scenarios.nova import networks
from tests.unit import test


class NovaNetworksTestCase(test.TestCase):

    def test_create_and_list_networks(self):
        scenario = networks.CreateAndListNetworks()

        scenario._create_network = mock.MagicMock()
        scenario._list_networks = mock.MagicMock()
        scenario._list_networks.return_value = ["fake_network_uuid",
                                                "new_network_uuid",
                                                "first_intergalactic_network"]
        start_cidr = "10.2.0.0/24"

        # Positive case:
        scenario._create_network.return_value = "new_network_uuid"

        scenario.run(start_cidr=start_cidr, fakearg="fakearg")

        scenario._create_network.assert_called_once_with(
            start_cidr, fakearg="fakearg")
        scenario._list_networks.assert_called_once_with()

        # Negative case 1: network isn't created
        scenario._create_network.return_value = None

        self.assertRaises(exceptions.RallyAssertionError,
                          scenario.run,
                          start_cidr=start_cidr, fakearg="fakearg")

        scenario._create_network.assert_called_with(
            start_cidr, fakearg="fakearg")

        # Negative case 2:  new network isn't in the list of available networks
        scenario._create_network.return_value = "universal_network"

        self.assertRaises(exceptions.RallyAssertionError,
                          scenario.run,
                          start_cidr=start_cidr, fakearg="fakearg")

        scenario._create_network.assert_called_with(
            start_cidr, fakearg="fakearg")
        scenario._list_networks.assert_called_with()

    def test_create_and_delete_network(self):
        scenario = networks.CreateAndDeleteNetwork()
        fake_network = mock.MagicMock()
        fake_network.cidr = "10.2.0.0/24"
        start_cidr = "10.2.0.0/24"
        scenario._create_network = mock.MagicMock(return_value=fake_network)
        scenario._delete_network = mock.MagicMock()
        scenario.run(start_cidr=start_cidr, fakearg="fakearg")

        scenario._create_network.assert_called_once_with(
            start_cidr, fakearg="fakearg")
        scenario._delete_network.assert_called_once_with(
            fake_network)



# Copyright 2016 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from rally import consts
from rally import exceptions
from rally.plugins.openstack import scenario
from rally.plugins.openstack.scenarios.nova import utils
from rally.task import types
from rally.task import validation


"""Scenarios for Nova aggregates."""


@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(admin=True)
@scenario.configure(name="NovaAggregates.list_aggregates")
class ListAggregates(utils.NovaScenario):

    def run(self):
        """List all nova aggregates.

        Measure the "nova aggregate-list" command performance.
        """
        self._list_aggregates()


@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["nova"]},
                    name="NovaAggregates.create_and_list_aggregates")
class CreateAndListAggregates(utils.NovaScenario):
    """scenario for create and list aggregate."""

    def run(self, availability_zone):
        """Create a aggregate and then list all aggregates.

        This scenario creates a aggregate and then lists all aggregates.
        :param availability_zone: The availability zone of the aggregate
        """
        aggregate = self._create_aggregate(availability_zone)
        msg = "Aggregate isn't created"
        self.assertTrue(aggregate, err_msg=msg)
        all_aggregates = self._list_aggregates()
        msg = ("Created aggregate is not in the"
               " list of all available aggregates")
        self.assertIn(aggregate, all_aggregates, err_msg=msg)


@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["nova"]},
                    name="NovaAggregates.create_and_delete_aggregate")
class CreateAndDeleteAggregate(utils.NovaScenario):
    """Scenario for create and delete aggregate."""

    def run(self, availability_zone):
        """Create an aggregate and then delete it.

        This scenario first creates an aggregate and then delete it.
        :param availability_zone: The availability zone of the aggregate
        """
        aggregate = self._create_aggregate(availability_zone)
        self._delete_aggregate(aggregate)


@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["nova"]},
                    name="NovaAggregates.create_and_update_aggregate")
class CreateAndUpdateAggregate(utils.NovaScenario):
    """Scenario for create and update aggregate."""

    def run(self, availability_zone):
        """Create an aggregate and then update its name and availability_zone

        This scenario first creates an aggregate and then update its name and
        availability_zone
        :param availability_zone: The availability zone of the aggregate
        """
        aggregate = self._create_aggregate(availability_zone)
        self._update_aggregate(aggregate)


@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["nova"]},
                    name="NovaAggregates.create_aggregate_add_and_remove_host")
class CreateAggregateAddAndRemoveHost(utils.NovaScenario):
    """Scenario for add a host to and remove the host from an aggregate."""

    def run(self, availability_zone):
        """Create an aggregate, add a host to and remove the host from it

        Measure "nova aggregate-add-host" and "nova aggregate-remove-host"
        command performance.
        :param availability_zone: The availability zone of the aggregate
        """
        aggregate = self._create_aggregate(availability_zone)
        hosts = self._list_hypervisors()
        host_name = hosts[0].service["host"]
        self._aggregate_add_host(aggregate, host_name)
        self._aggregate_remove_host(aggregate, host_name)


@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["nova"]},
                    name="NovaAggregates.create_and_get_aggregate_details")
class CreateAndGetAggregateDetails(utils.NovaScenario):
    """Scenario for create and get aggregate details."""

    def run(self, availability_zone):
        """Create an aggregate and then get its details.

        This scenario first creates an aggregate and then get details of it.
        :param availability_zone: The availability zone of the aggregate
        """
        aggregate = self._create_aggregate(availability_zone)
        self._get_aggregate_details(aggregate)


@types.convert(image={"type": "glance_image"})
@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(admin=True, users=True)
@scenario.configure(context={"admin_cleanup": ["nova"], "cleanup": ["nova"]},
                    name="NovaAggregates."
                         "create_aggregate_add_host_and_boot_server")
class CreateAggregateAddHostAndBootServer(utils.NovaScenario):
    """Scenario to verify an aggregate."""

    def run(self, image, metadata, availability_zone=None, ram=512, vcpus=1,
            disk=1, boot_server_kwargs=None):
        """Scenario to create and verify an aggregate

        This scenario creates an aggregate, adds a compute host and metadata
        to the aggregate, adds the same metadata to the flavor and creates an
        instance. Verifies that instance host is one of the hosts in the
        aggregate.

        :param image: The image ID to boot from
        :param metadata: The metadata to be set as flavor extra specs
        :param availability_zone: The availability zone of the aggregate
        :param ram: Memory in MB for the flavor
        :param vcpus: Number of VCPUs for the flavor
        :param disk: Size of local disk in GB
        :param boot_server_kwargs: Optional additional arguments to verify host
        aggregates
        :raises RallyException: if instance and aggregate hosts do not match
        """

        boot_server_kwargs = boot_server_kwargs or {}

        aggregate = self._create_aggregate(availability_zone)
        hosts = self._list_hypervisors()
        host_name = hosts[0].service["host"]
        self._aggregate_set_metadata(aggregate, metadata)
        self._aggregate_add_host(aggregate, host_name)
        flavor = self._create_flavor(ram, vcpus, disk)
        flavor.set_keys(metadata)

        server = self._boot_server(image, flavor.id, **boot_server_kwargs)
        # NOTE: we need to get server object by admin user to obtain
        # "hypervisor_hostname" attribute
        server = self.admin_clients("nova").servers.get(server.id)
        instance_hostname = getattr(server,
                                    "OS-EXT-SRV-ATTR:hypervisor_hostname")
        if instance_hostname != host_name:
            raise exceptions.RallyException("Instance host and aggregate "
                                            "host are different")



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import six

from rally.common.i18n import _
from rally import consts
from rally import exceptions
from rally.plugins.openstack import scenario
from rally.plugins.openstack.scenarios.nova import utils
from rally.task import atomic
from rally.task import types
from rally.task import validation


"""Scenarios for Nova security groups."""


class NovaSecurityGroupException(exceptions.RallyException):
    msg_fmt = _("%(message)s")


@validation.required_parameters("security_group_count",
                                "rules_per_security_group")
@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(users=True)
@scenario.configure(context={"cleanup": ["nova"]},
                    name="NovaSecGroup.create_and_delete_secgroups")
class CreateAndDeleteSecgroups(utils.NovaScenario):

    def run(self, security_group_count, rules_per_security_group):
        """Create and delete security groups.

        This scenario creates N security groups with M rules per group and then
        deletes them.

        :param security_group_count: Number of security groups
        :param rules_per_security_group: Number of rules per security group
        """

        security_groups = self._create_security_groups(
            security_group_count)

        self._create_rules_for_security_group(security_groups,
                                              rules_per_security_group)

        self._delete_security_groups(security_groups)


@validation.required_parameters("security_group_count",
                                "rules_per_security_group")
@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(users=True)
@scenario.configure(context={"cleanup": ["nova"]},
                    name="NovaSecGroup.create_and_list_secgroups")
class CreateAndListSecgroups(utils.NovaScenario):

    def run(self, security_group_count, rules_per_security_group):
        """Create and list security groups.

        This scenario creates N security groups with M rules per group and then
        lists them.

        :param security_group_count: Number of security groups
        :param rules_per_security_group: Number of rules per security group
        """

        security_groups = self._create_security_groups(
            security_group_count)

        self.assertTrue(security_groups)

        self._create_rules_for_security_group(security_groups,
                                              rules_per_security_group)

        pool_groups = self._list_security_groups()
        self.assertLessEqual(len(security_groups), len(pool_groups))
        self.assertIsSubset([i.id for i in security_groups],
                            [i.id for i in pool_groups])


@validation.required_parameters("security_group_count")
@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(users=True)
@scenario.configure(context={"cleanup": ["nova"]},
                    name="NovaSecGroup.create_and_update_secgroups")
class CreateAndUpdateSecgroups(utils.NovaScenario):

    def run(self, security_group_count):
        """Create and update security groups.

        This scenario creates 'security_group_count' security groups
        then updates their name and description.

        :param security_group_count: Number of security groups
        """
        security_groups = self._create_security_groups(
            security_group_count)
        self._update_security_groups(security_groups)


@types.convert(image={"type": "glance_image"},
               flavor={"type": "nova_flavor"})
@validation.image_valid_on_flavor("flavor", "image")
@validation.required_parameters("security_group_count",
                                "rules_per_security_group")
@validation.required_contexts("network")
@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(users=True)
@scenario.configure(context={"cleanup": ["nova"]},
                    name="NovaSecGroup.boot_and_delete_server_with_secgroups")
class BootAndDeleteServerWithSecgroups(utils.NovaScenario):

    def run(self, image, flavor, security_group_count,
            rules_per_security_group, **kwargs):
        """Boot and delete server with security groups attached.

        Plan of this scenario:
         - create N security groups with M rules per group
           vm with security groups)
         - boot a VM with created security groups
         - get list of attached security groups to server
         - delete server
         - delete all security groups
         - check that all groups were attached to server

        :param image: ID of the image to be used for server creation
        :param flavor: ID of the flavor to be used for server creation
        :param security_group_count: Number of security groups
        :param rules_per_security_group: Number of rules per security group
        :param **kwargs: Optional arguments for booting the instance
        """

        security_groups = self._create_security_groups(
            security_group_count)
        self._create_rules_for_security_group(security_groups,
                                              rules_per_security_group)

        secgroups_names = [sg.name for sg in security_groups]
        server = self._boot_server(image, flavor,
                                   security_groups=secgroups_names,
                                   **kwargs)

        action_name = "nova.get_attached_security_groups"
        with atomic.ActionTimer(self, action_name):
            attached_security_groups = server.list_security_group()

        self._delete_server(server)
        try:
            self._delete_security_groups(security_groups)
        except Exception as e:
            if hasattr(e, "http_status") and e.http_status == 400:
                raise NovaSecurityGroupException(six.text_type(e))
            raise

        error_message = ("Expected number of attached security groups to "
                         " server %(server)s is '%(all)s', but actual number "
                         "is '%(attached)s'." % {
                             "attached": len(attached_security_groups),
                             "all": len(security_groups),
                             "server": server})

        self.assertEqual(sorted([sg.id for sg in security_groups]),
                         sorted([sg.id for sg in attached_security_groups]),
                         error_message)


@types.convert(image={"type": "glance_image"},
               flavor={"type": "nova_flavor"})
@validation.image_valid_on_flavor("flavor", "image")
@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(users=True)
@validation.required_contexts("network")
@scenario.configure(context={"cleanup": ["nova"]},
                    name="NovaSecGroup.boot_server_and_add_secgroups")
class BootServerAndAddSecgroups(utils.NovaScenario):
    def run(self, image, flavor, security_group_count=1,
            rules_per_security_group=1, **kwargs):
        """Boot a server and add a security group to it.

        Plan of this scenario:
         - create N security groups with M rules per group
         - boot a VM
         - add security groups to VM

        :param image: ID of the image to be used for server creation
        :param flavor: ID of the flavor to be used for server creation
        :param security_group_count: Number of security groups
        :param rules_per_security_group: Number of rules per security group
        :param **kwargs: Optional arguments for booting the instance
        """
        server = self._boot_server(image, flavor, **kwargs)
        security_groups = self._create_security_groups(security_group_count)
        self._create_rules_for_security_group(security_groups,
                                              rules_per_security_group)

        with atomic.ActionTimer(self, "nova.add_server_secgroups"):
            for sg in security_groups:
                self._add_server_secgroups(server, sg.name,
                                           atomic_action=False)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Scenarios for Ceilometer Events API.
"""

from rally import consts
from rally.plugins.openstack import scenario
from rally.plugins.openstack.scenarios.ceilometer import utils as cutils
from rally.plugins.openstack.scenarios.keystone import basic as kbasic
from rally.task import validation


# NOTE(idegtiarov): to work with event we need to create it, there are
# no other way except emit suitable notification from one of services,
# for example create new user in keystone.

@validation.required_services(consts.Service.CEILOMETER,
                              consts.Service.KEYSTONE)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["keystone"],
                             "cleanup": ["ceilometer"]},
                    name="CeilometerEvents.create_user_and_list_events")
class CeilometerEventsCreateUserAndListEvents(cutils.CeilometerScenario,
                                              kbasic.KeystoneBasic):

    def run(self):
        """Create user and fetch all events.

        This scenario creates user to store new event and
        fetches list of all events using GET /v2/events.
        """
        self.admin_keystone.create_user()
        events = self._list_events()
        msg = ("Events list is empty, but it should include at least one "
               "event about user creation")
        self.assertTrue(events, msg)


@validation.required_services(consts.Service.CEILOMETER,
                              consts.Service.KEYSTONE)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["keystone"],
                             "cleanup": ["ceilometer"]},
                    name="CeilometerEvents.create_user_and_list_event_types")
class CeilometerEventsCreateUserAndListEventTypes(cutils.CeilometerScenario,
                                                  kbasic.KeystoneBasic):

    def run(self):
        """Create user and fetch all event types.

        This scenario creates user to store new event and
        fetches list of all events types using GET /v2/event_types.
        """
        self.admin_keystone.create_user()
        event_types = self._list_event_types()
        msg = ("Event types list is empty, but it should include at least one"
               " type about user creation")
        self.assertTrue(event_types, msg)


@validation.required_services(consts.Service.CEILOMETER,
                              consts.Service.KEYSTONE)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["keystone"],
                             "cleanup": ["ceilometer"]},
                    name="CeilometerEvents.create_user_and_get_event")
class CeilometerEventsCreateUserAndGetEvent(cutils.CeilometerScenario,
                                            kbasic.KeystoneBasic):

    def run(self):
        """Create user and gets event.

        This scenario creates user to store new event and
        fetches one event using GET /v2/events/<message_id>.
        """
        self.admin_keystone.create_user()
        events = self._list_events()
        msg = ("Events list is empty, but it should include at least one "
               "event about user creation")
        self.assertTrue(events, msg)
        self._get_event(event_id=events[0].message_id)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from rally.plugins.openstack import service
from tests.unit import test


class DiscoverTestCase(test.TestCase):
    def test_discover_network_impl_based_on_service(self):

        class SomeService(service.UnifiedOpenStackService):
            pass

        @service.service("nova-network", "network", version="1",
                         client_name="nova")
        class NovaNetService(service.Service):
            pass

        @service.compat_layer(NovaNetService)
        class UnifiedNovaNetService(SomeService):
            @classmethod
            def is_applicable(cls, clients):
                return True

        @service.service("neutron", "network", version="2")
        class NeutronV2Service(service.Service):
            pass

        @service.compat_layer(NeutronV2Service)
        class UnifiedNeutronV2Service(SomeService):
            pass

        clients = mock.MagicMock()
        clients.nova.choose_version.return_value = "1"
        clients.neutron.choose_version.return_value = "2"

        clients.services.return_value = {}
        self.assertIsInstance(SomeService(clients)._impl,
                              UnifiedNovaNetService)

        clients.nova.return_value.services.list.reset_mock()

        clients.services.return_value = {"network": "neutron"}
        self.assertIsInstance(SomeService(clients)._impl,
                              UnifiedNeutronV2Service)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from rally import exceptions
from rally.plugins.openstack.scenarios.nova import security_group
from tests.unit import fakes
from tests.unit import test


class FakeNeutronScenario(object):
    def __enter__(self):
        return {}

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass


class NovaSecurityGroupTestCase(test.ScenarioTestCase):

    def test_create_and_delete_security_groups(self):
        fake_secgroups = [fakes.FakeSecurityGroup(None, None, 1, "uuid1"),
                          fakes.FakeSecurityGroup(None, None, 2, "uuid2")]

        scenario = security_group.CreateAndDeleteSecgroups(self.context)
        scenario._create_security_groups = mock.MagicMock(
            return_value=fake_secgroups)
        scenario._create_rules_for_security_group = mock.MagicMock()
        scenario._delete_security_groups = mock.MagicMock()

        security_group_count = 2
        rules_per_security_group = 10
        scenario.run(security_group_count, rules_per_security_group)

        scenario._create_security_groups.assert_called_once_with(
            security_group_count)
        scenario._create_rules_for_security_group.assert_called_once_with(
            fake_secgroups, rules_per_security_group)
        scenario._delete_security_groups.assert_called_once_with(
            fake_secgroups)

    def test_create_and_update_security_groups(self):
        fake_secgroups = [fakes.FakeSecurityGroup(None, None, 1, "uuid1"),
                          fakes.FakeSecurityGroup(None, None, 2, "uuid2")]
        scenario = security_group.CreateAndUpdateSecgroups(self.context)
        scenario._create_security_groups = mock.MagicMock(
            return_value=fake_secgroups)
        scenario._update_security_groups = mock.MagicMock()
        scenario._generate_random_name = mock.Mock(
            return_value="_updated")
        security_group_count = 2
        scenario.run(security_group_count)
        scenario._create_security_groups.assert_called_once_with(
            security_group_count)
        scenario._update_security_groups.assert_called_once_with(
            fake_secgroups)

    def test_create_and_list_secgroups(self):
        fake_secgroups = [fakes.FakeSecurityGroup(None, None, 1, "uuid1"),
                          fakes.FakeSecurityGroup(None, None, 2, "uuid2")]

        scenario = security_group.CreateAndListSecgroups(self.context)

        scenario._create_security_groups = mock.MagicMock()
        scenario._create_rules_for_security_group = mock.MagicMock()
        scenario._list_security_groups = mock.MagicMock()

        scenario._list_security_groups.return_value = fake_secgroups
        scenario._list_security_groups.return_value.append(
            fakes.FakeSecurityGroup(None, None, 3, "uuid3"))
        scenario._list_security_groups.return_value.append(
            fakes.FakeSecurityGroup(None, None, 4, "uuid4"))

        security_group_count = 2
        rules_per_security_group = 10

        # Positive case:
        scenario._create_security_groups.return_value = fake_secgroups
        scenario.run(
            security_group_count, rules_per_security_group)

        scenario._create_security_groups.assert_called_once_with(
            security_group_count)
        scenario._create_rules_for_security_group.assert_called_once_with(
            fake_secgroups, rules_per_security_group)
        scenario._list_security_groups.assert_called_once_with()

        # Negative case1: groups aren't created
        scenario._create_security_groups.return_value = None
        self.assertRaises(exceptions.RallyAssertionError,
                          scenario.run,
                          security_group_count, rules_per_security_group)
        scenario._create_security_groups.assert_called_with(
            security_group_count)

        # Negative case2: new groups are not present in the list of groups
        fake_secgroups = [fakes.FakeSecurityGroup(None, None, 6, "uuid6")]
        scenario._create_security_groups.return_value = fake_secgroups
        scenario._create_rules_for_security_group = mock.MagicMock()
        self.assertRaises(exceptions.RallyAssertionError,
                          scenario.run,
                          security_group_count, rules_per_security_group)
        scenario._create_security_groups.assert_called_with(
            security_group_count)
        scenario._create_rules_for_security_group.assert_called_with(
            fake_secgroups, rules_per_security_group)
        scenario._list_security_groups.assert_called_with()

    def _generate_fake_server_with_sg(self, number_of_secgroups):
        sg_list = []
        for i in range(number_of_secgroups):
            sg_list.append(
                fakes.FakeSecurityGroup(None, None, i, "uuid%s" % i))

        return mock.MagicMock(
            list_security_group=mock.MagicMock(return_value=sg_list)), sg_list

    def _test_boot_and_delete_server_with_secgroups(self):
        fake_server, sg_list = self._generate_fake_server_with_sg(2)

        scenario = security_group.BootAndDeleteServerWithSecgroups(
            self.context)
        scenario._create_security_groups = mock.MagicMock(
            return_value=sg_list)
        scenario._create_rules_for_security_group = mock.MagicMock()
        scenario._boot_server = mock.MagicMock(return_value=fake_server)
        scenario.generate_random_name = mock.MagicMock(
            return_value="name")
        scenario._delete_server = mock.MagicMock()
        scenario._delete_security_groups = mock.MagicMock()

        image = "img"
        flavor = 1
        security_group_count = 2
        rules_per_security_group = 10

        scenario.run(
            image, flavor, security_group_count, rules_per_security_group,
            fakearg="fakearg")
        scenario._create_security_groups.assert_called_once_with(
            security_group_count)
        scenario.generate_random_name.assert_called_once_with()
        scenario._create_rules_for_security_group.assert_called_once_with(
            sg_list, rules_per_security_group)
        scenario._boot_server.assert_called_once_with(
            "name", image, flavor,
            security_groups=[sg.name for sg in sg_list], fakearg="fakearg")
        fake_server.list_security_group.assert_called_once_with()
        scenario._delete_server.assert_called_once_with(fake_server)
        scenario._delete_security_groups.assert_called_once_with(sg_list)

    def _test_boot_and_delete_server_with_sg_not_attached(self):
        fake_secgroups = [fakes.FakeSecurityGroup(None, None, 1, "uuid1"),
                          fakes.FakeSecurityGroup(None, None, 2, "uuid2")]

        fake_server, sg_list = self._generate_fake_server_with_sg(1)

        scenario = security_group.BootAndDeleteServerWithSecgroups()
        scenario._create_security_groups = mock.MagicMock(
            return_value=fake_secgroups)
        scenario._create_rules_for_security_group = mock.MagicMock()
        scenario._boot_server = mock.MagicMock(return_value=fake_server)
        scenario.generate_random_name = mock.MagicMock(
            return_value="name")
        scenario._delete_server = mock.MagicMock()
        scenario._delete_security_groups = mock.MagicMock()

        image = "img"
        flavor = 1
        security_group_count = 2
        rules_per_security_group = 10

        self.assertRaises(security_group.NovaSecurityGroupException,
                          scenario.run,
                          image, flavor, security_group_count,
                          rules_per_security_group)

        scenario._create_security_groups.assert_called_once_with(
            security_group_count)
        scenario.generate_random_name.assert_called_once_with()
        scenario._create_rules_for_security_group.assert_called_once_with(
            fake_secgroups, rules_per_security_group)
        scenario._boot_server.assert_called_once_with(
            "name", image, flavor,
            security_groups=[sg.name for sg in fake_secgroups])
        fake_server.list_security_group.assert_called_once_with()
        scenario._delete_server.assert_called_once_with(fake_server)
        scenario._delete_security_groups.assert_called_once_with(
            fake_secgroups)

    def test_boot_server_and_add_secgroups(self):
        fake_secgroups = [fakes.FakeSecurityGroup(None, None, 1, "uuid1"),
                          fakes.FakeSecurityGroup(None, None, 2, "uuid2")]

        nova_scenario = security_group.BootServerAndAddSecgroups(self.context)
        nova_scenario._create_security_groups = mock.MagicMock(
            return_value=fake_secgroups)
        nova_scenario._create_rules_for_security_group = mock.MagicMock()
        nova_scenario._boot_server = mock.MagicMock()
        nova_scenario.add_server_secgroup = mock.MagicMock()

        image = "img"
        flavor = 1
        security_group_count = 2
        rules_per_security_group = 10

        nova_scenario.run(
            image, flavor, security_group_count, rules_per_security_group,
            fakearg="fakearg")

        nova_scenario._create_security_groups.assert_called_once_with(
            security_group_count)
        nova_scenario._create_rules_for_security_group.assert_called_once_with(
            fake_secgroups, rules_per_security_group)
        nova_scenario._boot_server.assert_called_once_with(image, flavor,
                                                           fakearg="fakearg")



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse
import gzip
import json
import logging
import os
import re
import subprocess
import sys
import uuid

from rally.cli import envutils
from rally.common import objects
from rally import osclients
from rally.ui import utils

LOG = logging.getLogger(__name__)
LOG.setLevel(logging.DEBUG)

BASE_DIR = "rally-verify"

MODES = {"full": "--pattern set=full", "light": "--pattern set=smoke"}
DEPLOYMENT_NAME = "devstack"
VERIFIER_TYPE = "tempest"
VERIFIER_SOURCE = "https://git.openstack.org/openstack/tempest"
VERIFIER_EXT_REPO = (
    "https://git.openstack.org/openstack/keystone-tempest-plugin")
VERIFIER_EXT_NAME = "keystone_tests"
SKIP_TESTS = (
    "tempest.api.compute.flavors.test_flavors.FlavorsV2TestJSON."
    "test_get_flavor[id-1f12046b-753d-40d2-abb6-d8eb8b30cb2f,smoke]: "
    "This test was skipped intentionally")
XFAIL_TESTS = (
    "tempest.api.compute.servers.test_server_actions.ServerActionsTestJSON."
    "test_get_vnc_console[id-c6bc11bf-592e-4015-9319-1c98dc64daf5]: "
    "This test fails because 'novnc' console type is unavailable")
TEST_NAME_RE = re.compile(r"^[a-zA-Z_.0-9]+(\[[a-zA-Z-,=0-9]*\])?$")

# NOTE(andreykurilin): this variable is used to generate output file names
# with prefix ${CALL_COUNT}_ .
_call_count = 0
# NOTE(andreykurilin): if some command fails, script should end with
# error status
_return_status = 0


def call_rally(cmd, print_output=False, output_type=None):
    """Execute a Rally command and write result in files."""
    global _return_status
    global _call_count
    _call_count += 1

    data = {"cmd": "rally --rally-debug %s" % cmd}
    stdout_file = "{base_dir}/{prefix}_{cmd}.txt.gz"

    cmd = cmd.replace("/", "_")
    data.update({"stdout_file": stdout_file.format(base_dir=BASE_DIR,
                                                   prefix=_call_count,
                                                   cmd=cmd.replace(" ", "_"))})

    if output_type:
        data["output_file"] = data["stdout_file"].replace(
            ".txt.", ".%s." % output_type)
        data["cmd"] += " --to %s" % data["output_file"]
        data["cmd"] += " --type %s" % output_type

    try:
        LOG.info("Try to execute `%s`." % data["cmd"])
        stdout = subprocess.check_output(data["cmd"].split(),
                                         stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        LOG.error("Command `%s` failed." % data["cmd"])
        stdout = e.output
        data["status"] = "fail"
        _return_status = 1
    else:
        data["status"] = "success"

    if output_type:
        # let's gzip results
        with open(data["output_file"]) as f:
            output = f.read()
        with gzip.open(data["output_file"], "wb") as f:
            f.write(output)

    stdout = "$ %s\n%s" % (data["cmd"], stdout)

    with gzip.open(data["stdout_file"], "wb") as f:
        f.write(stdout)

    if print_output:
        print(stdout)

    return data


def start_verification(args):
    """Start a verification, show results and generate reports."""
    results = call_rally("verify start %s" % args)
    results["uuid"] = envutils.get_global(envutils.ENV_VERIFICATION)
    results["show"] = call_rally("verify show")
    results["show_detailed"] = call_rally("verify show --detailed")
    for output_type in ("json", "html", "junit-xml"):
        results[output_type.replace("-", "_")] = call_rally(
            "verify report", output_type=output_type)
    # NOTE(andreykurilin): we need to clean verification uuid from global
    # environment to be able to load it next time(for another verification).
    envutils.clear_global(envutils.ENV_VERIFICATION)
    return results


def write_file(filename, data):
    """Create a file and write some data to it."""
    path = os.path.join(BASE_DIR, filename)
    with open(path, "wb") as f:
        f.write(data)
    return path


def generate_trends_reports(uuid_1, uuid_2):
    """Generate trends reports."""
    results = {}
    for output_type in ("json", "html", "junit-xml"):
        results[output_type.replace("-", "_")] = call_rally(
            "verify report --uuid %s %s" % (uuid_1, uuid_2),
            output_type=output_type)
    return results


def render_page(**render_vars):
    template = utils.get_template("ci/index_verify.html")
    with open(os.path.join(BASE_DIR, "extra/index.html"), "w") as f:
        f.write(template.render(**render_vars))


def main():
    parser = argparse.ArgumentParser(description="Launch rally-verify job.")
    parser.add_argument("--mode", type=str, default="light",
                        help="Mode of job. The 'full' mode corresponds to the "
                             "full set of verifier tests. The 'light' mode "
                             "corresponds to the smoke set of verifier tests.",
                        choices=MODES.keys())
    parser.add_argument("--compare", action="store_true",
                        help="Start the second verification to generate a "
                             "trends report for two verifications.")
    # TODO(ylobankov): Remove hard-coded Tempest related things and make it
    #                  configurable.
    parser.add_argument("--ctx-create-resources", action="store_true",
                        help="Make Tempest context create needed resources "
                             "for the tests.")

    args = parser.parse_args()

    if not os.path.exists("%s/extra" % BASE_DIR):
        os.makedirs("%s/extra" % BASE_DIR)

    # Choose and check the deployment
    call_rally("deployment use --deployment %s" % DEPLOYMENT_NAME)
    call_rally("deployment check")

    config = json.loads(
        subprocess.check_output(["rally", "deployment", "config"]))
    config.update(config.pop("admin"))
    del config["type"]
    clients = osclients.Clients(objects.Credential(**config))

    if args.ctx_create_resources:
        # If the 'ctx-create-resources' arg is provided, delete images and
        # flavors, and also create a shared network to make Tempest context
        # create needed resources.
        LOG.info("The 'ctx-create-resources' arg is provided. Deleting "
                 "images and flavors, and also creating a shared network "
                 "to make Tempest context create needed resources.")

        LOG.info("Deleting images.")
        for image in clients.glance().images.list():
            clients.glance().images.delete(image.id)

        LOG.info("Deleting flavors.")
        for flavor in clients.nova().flavors.list():
            clients.nova().flavors.delete(flavor.id)

        LOG.info("Creating a shared network.")
        net_body = {
            "network": {
                "name": "shared-net-%s" % str(uuid.uuid4()),
                "tenant_id": clients.keystone.auth_ref.project_id,
                "shared": True
            }
        }
        clients.neutron().create_network(net_body)
    else:
        # Otherwise, just in case create only flavors with the following
        # properties: RAM = 64MB and 128MB, VCPUs = 1, disk = 0GB to make
        # Tempest context discover them.
        LOG.info("The 'ctx-create-resources' arg is not provided. "
                 "Creating flavors to make Tempest context discover them.")
        for flv_ram in [64, 128]:
            params = {
                "name": "flavor-%s" % str(uuid.uuid4()),
                "ram": flv_ram,
                "vcpus": 1,
                "disk": 0
            }
            LOG.info(
                "Creating flavor '%s' with the following properties: RAM "
                "= %dMB, VCPUs = 1, disk = 0GB" % (params["name"], flv_ram))
            clients.nova().flavors.create(**params)

    render_vars = dict(verifications=[])

    # List plugins for verifiers management
    render_vars["list_plugins"] = call_rally("verify list-plugins")

    # Create a verifier
    render_vars["create_verifier"] = call_rally(
        "verify create-verifier --type %s --name my-verifier --source %s"
        % (VERIFIER_TYPE, VERIFIER_SOURCE))

    # Show the verifier
    render_vars["show_verifier"] = call_rally("verify show-verifier")

    # List verifiers
    render_vars["list_verifiers"] = call_rally("verify list-verifiers")

    # Get verifier ID
    verifier_id = envutils.get_global(envutils.ENV_VERIFIER)
    # Get the penultimate verifier commit ID
    repo_dir = os.path.join(
        os.path.expanduser("~"),
        ".rally/verification/verifier-%s/repo" % verifier_id)
    p_commit_id = subprocess.check_output(
        ["git", "log", "-n", "1", "--pretty=format:%H"], cwd=repo_dir).strip()
    # Switch the verifier to the penultimate version
    render_vars["update_verifier"] = call_rally(
        "verify update-verifier --version %s --update-venv" % p_commit_id)

    # Generate and show the verifier config file
    render_vars["configure_verifier"] = call_rally(
        "verify configure-verifier --show")

    # Add a verifier extension
    render_vars["add_verifier_ext"] = call_rally(
        "verify add-verifier-ext --source %s" % VERIFIER_EXT_REPO)

    # List verifier extensions
    render_vars["list_verifier_exts"] = call_rally("verify list-verifier-exts")

    # List verifier tests
    render_vars["list_verifier_tests"] = call_rally(
        "verify list-verifier-tests %s" % MODES[args.mode])

    # Start a verification, show results and generate reports
    skip_list_path = write_file("skip-list.yaml", SKIP_TESTS)
    xfail_list_path = write_file("xfail-list.yaml", XFAIL_TESTS)
    run_args = ("%s --skip-list %s --xfail-list %s --tag first-run %s-set "
                "--detailed" % (MODES[args.mode], skip_list_path,
                                xfail_list_path, args.mode))
    render_vars["verifications"].append(start_verification(run_args))

    if args.compare:
        # Start another verification, show results and generate reports
        with gzip.open(render_vars["list_verifier_tests"]["stdout_file"]) as f:
            tests = [t for t in f.read().split("\n") if TEST_NAME_RE.match(t)]
            load_list_path = write_file("load-list.txt", "\n".join(tests))
        run_args = "--load-list %s --tag second-run %s-set --detailed" % (
            load_list_path, args.mode)
        render_vars["verifications"].append(start_verification(run_args))

        # Generate trends reports for two verifications
        render_vars["compare"] = generate_trends_reports(
            render_vars["verifications"][-2]["uuid"],
            render_vars["verifications"][-1]["uuid"])

    # List verifications
    render_vars["list"] = call_rally("verify list")

    # Delete the verifier extension
    render_vars["delete_verifier_ext"] = call_rally(
        "verify delete-verifier-ext --name %s" % VERIFIER_EXT_NAME)
    # Delete the verifier and all verifications
    render_vars["delete_verifier"] = call_rally(
        "verify delete-verifier --id %s --force" % verifier_id)

    render_page(**render_vars)

    return _return_status

if __name__ == "__main__":
    sys.exit(main())



# Copyright 2013: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Test ExistingCloud."""

import ddt
import jsonschema

from rally import consts
from rally.deployment import engine as deploy_engine
from rally.deployment.engines import existing
from tests.unit import test


@ddt.ddt
class TestExistingCloud(test.TestCase):
    def setUp(self):
        super(TestExistingCloud, self).setUp()
        self.deployments = {
            "v2.0": {
                "config": {
                    "type": "ExistingCloud",
                    "auth_url": "http://example.net:5000/v2.0/",
                    "region_name": "RegionOne",
                    "endpoint_type": consts.EndpointType.INTERNAL,
                    "https_insecure": False,
                    "https_cacert": "cacert",
                    "admin": {
                        "username": "admin",
                        "password": "myadminpass",
                        "tenant_name": "demo"
                    }
                }
            },
            "v3": {
                "config": {
                    "type": "ExistingCloud",
                    "auth_url": "http://example.net:5000/v3/",
                    "region_name": "RegionOne",
                    "endpoint_type": consts.EndpointType.INTERNAL,
                    "https_insecure": False,
                    "https_cacert": "cacert",
                    "admin": {
                        "username": "admin",
                        "password": "myadminpass",
                        "domain_name": "domain",
                        "project_name": "demo",
                        "project_domain_name": "Default",
                        "user_domain_name": "Default",
                    }
                }
            }
        }

    @ddt.data("v2.0", "v3")
    def test_init_and_valid_config(self, keystone_version):
        engine = existing.ExistingCloud(self.deployments[keystone_version])
        engine.validate()

    @ddt.data("v2.0", "v3")
    def test_invalid_config(self, keystone_version):
        deployment = self.deployments[keystone_version]
        deployment["config"]["admin"] = 42
        engine = existing.ExistingCloud(deployment)
        self.assertRaises(jsonschema.ValidationError,
                          engine.validate)

    @ddt.data("v2.0", "v3")
    def test_additional_vars(self, keystone_version):
        deployment = self.deployments[keystone_version]
        deployment["extra"] = {}
        existing.ExistingCloud(deployment).validate()

        deployment["extra"] = {"some_var": "some_value"}
        existing.ExistingCloud(deployment).validate()

        deployment["extra"] = ["item1", "item2"]
        existing.ExistingCloud(deployment).validate()

    @ddt.data("v2.0", "v3")
    def test_deploy(self, keystone_version):
        deployment = self.deployments[keystone_version]
        engine = existing.ExistingCloud(deployment)
        credentials = engine.deploy()
        credentials = credentials["openstack"][0]
        admin_credential = deployment["config"].copy()
        admin_credential.pop("type")
        admin_credential["endpoint"] = None
        admin_credential.update(admin_credential.pop("admin"))
        admin_credential["permission"] = consts.EndpointPermission.ADMIN

        actual_credentials = credentials["admin"]

        if keystone_version == "v3":
            # NOTE(andreykurilin): credentials obj uses `tenant_name` for both
            #   keystone v2 and v3. It works perfectly for rally code (no
            #   contradictions and misunderstandings ), but in case of checking
            #   credentials.to_dict with data from database (where we use
            #   project_name for keystone v3 config and tenant_name for
            #   keystone v2), we need to transform vars.
            admin_credential["tenant_name"] = admin_credential.pop(
                "project_name")
        else:
            # NOTE(andreykurilin): there are no domain related variables in v2,
            #   so we need to pop them from credentials.to_dict()
            actual_credentials.pop("domain_name")
            actual_credentials.pop("user_domain_name")
            actual_credentials.pop("project_domain_name")

        self.assertEqual(admin_credential, actual_credentials)
        self.assertEqual([], credentials["users"])

    @ddt.data("v2.0", "v3")
    def test_cleanup(self, keystone_version):
        existing.ExistingCloud(self.deployments[keystone_version]).cleanup()

    @ddt.data("v2.0", "v3")
    def test_is_in_factory(self, keystone_version):
        name = self.deployments[keystone_version]["config"]["type"]
        engine = deploy_engine.Engine.get_engine(
            name, self.deployments[keystone_version])
        self.assertIsInstance(engine, existing.ExistingCloud)



# Copyright 2016 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from rally import consts
from rally.plugins.openstack import scenario
from rally.plugins.openstack.scenarios.nova import utils
from rally.task import validation


"""Scenarios for Nova availability-zones."""


@validation.required_services(consts.Service.NOVA)
@validation.required_openstack(admin=True)
@scenario.configure(name="NovaAvailabilityZones.list_availability_zones")
class ListAvailabilityZones(utils.NovaScenario):

    def run(self, detailed=True):
        """List all availability zones.

        Measure the "nova availability-zone-list" command performance.

        :param detailed: True if the availability-zone listing should contain
                         detailed information about all of them
        """
        self._list_availability_zones(detailed)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from rally import consts
from rally.plugins.openstack import scenario
from rally.plugins.openstack.scenarios.ceilometer import utils as cutils
from rally.plugins.openstack.scenarios.keystone import basic as kbasic
from rally.task import validation


"""Scenarios for Ceilometer Events API."""


# NOTE(idegtiarov): to work with traits we need to create event firstly,
# there are no other way except emit suitable notification from one of
# services, for example create new user in keystone.

@validation.required_services(consts.Service.CEILOMETER,
                              consts.Service.KEYSTONE)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["keystone"],
                             "cleanup": ["ceilometer"]},
                    name="CeilometerTraits.create_user_and_list_traits")
class CreateUserAndListTraits(cutils.CeilometerScenario,
                              kbasic.KeystoneBasic):

    def run(self):
        """Create user and fetch all event traits.

        This scenario creates user to store new event and
        fetches list of all traits for certain event type and
        trait name using GET /v2/event_types/<event_type>/traits/<trait_name>.
        """
        self.admin_keystone.create_user()
        event = self._list_events()[0]
        trait_name = event.traits[0]["name"]
        self._list_event_traits(event_type=event.event_type,
                                trait_name=trait_name)


@validation.required_services(consts.Service.CEILOMETER,
                              consts.Service.KEYSTONE)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["keystone"],
                             "cleanup": ["ceilometer"]},
                    name="CeilometerTraits.create_user_and"
                         "_list_trait_descriptions")
class CreateUserAndListTraitDescriptions(
        cutils.CeilometerScenario, kbasic.KeystoneBasic):

    def run(self):
        """Create user and fetch all trait descriptions.

        This scenario creates user to store new event and
        fetches list of all traits for certain event type using
        GET /v2/event_types/<event_type>/traits.
        """
        self.admin_keystone.create_user()
        event = self._list_events()[0]
        self._list_event_trait_descriptions(event_type=event.event_type)



# Copyright 2013 Huawei Technologies Co.,LTD.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import mock

from rally.plugins.openstack.scenarios.cinder import volumes
from tests.unit import test

CINDER_VOLUMES = ("rally.plugins.openstack.scenarios.cinder.volumes"
                  ".CinderVolumes")


class fake_type(object):
    name = "fake"


@ddt.ddt
class CinderServersTestCase(test.ScenarioTestCase):

    def _get_context(self):
        context = test.get_test_context()
        context.update({
            "user": {"tenant_id": "fake",
                     "credential": mock.MagicMock()},
            "tenant": {"id": "fake", "name": "fake",
                       "volumes": [{"id": "uuid", "size": 1}],
                       "servers": [1]}})
        return context

    def test_create_and_list_volume(self):
        scenario = volumes.CreateAndListVolume(self.context)
        scenario._create_volume = mock.MagicMock()
        scenario._list_volumes = mock.MagicMock()
        scenario.run(1, True, fakearg="f")
        scenario._create_volume.assert_called_once_with(1, fakearg="f")
        scenario._list_volumes.assert_called_once_with(True)

    def test_create_and_get_volume(self):
        scenario = volumes.CreateAndGetVolume(self.context)
        scenario._create_volume = mock.MagicMock()
        scenario._get_volume = mock.MagicMock()
        scenario.run(1, fakearg="f")
        scenario._create_volume.assert_called_once_with(1, fakearg="f")
        scenario._get_volume.assert_called_once_with(
            scenario._create_volume.return_value.id)

    def test_list_volumes(self):
        scenario = volumes.ListVolumes(self.context)
        scenario._list_volumes = mock.MagicMock()
        scenario.run(True)
        scenario._list_volumes.assert_called_once_with(True)

    def test_list_types(self):
        scenario = volumes.ListTypes(self.context)
        scenario._list_types = mock.MagicMock()
        scenario.run(None, None)
        scenario._list_types.assert_called_once_with(None, None)

    def test_list_transfers(self):
        scenario = volumes.ListTransfers(self.context)
        scenario._list_transfers = mock.MagicMock()
        scenario.run(True, None)
        scenario._list_transfers.assert_called_once_with(True, None)

    def test_create_and_update_volume(self):
        volume_update_args = {"dispaly_name": "_updated"}
        scenario = volumes.CreateAndUpdateVolume(self.context)
        fake_volume = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._update_volume = mock.MagicMock()
        scenario.run(1, update_volume_kwargs=volume_update_args)
        scenario._create_volume.assert_called_once_with(1)
        scenario._update_volume.assert_called_once_with(fake_volume,
                                                        **volume_update_args)

    def test_create_volume_and_update_readonly_flag(self):
        scenario = volumes.CreateVolumeAndUpdateReadonlyFlag(self.context)
        fake_volume = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._update_readonly_flag = mock.MagicMock()
        scenario.run(1, None, True, fakearg="f")
        scenario._create_volume.assert_called_once_with(1, fakearg="f")
        scenario._update_readonly_flag.assert_called_once_with(
            fake_volume.id, True)

    def test_create_and_delete_volume(self):
        fake_volume = mock.MagicMock()

        scenario = volumes.CreateAndDeleteVolume(self.context)
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario.sleep_between = mock.MagicMock()
        scenario._delete_volume = mock.MagicMock()

        scenario.run(size=1, min_sleep=10, max_sleep=20, fakearg="f")

        scenario._create_volume.assert_called_once_with(1, fakearg="f")
        scenario.sleep_between.assert_called_once_with(10, 20)
        scenario._delete_volume.assert_called_once_with(fake_volume)

    def test_create_volume(self):
        fake_volume = mock.MagicMock()
        scenario = volumes.CreateVolume(self.context)
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)

        scenario.run(1, fakearg="f")
        scenario._create_volume.assert_called_once_with(1, fakearg="f")

    def test_create_volume_and_modify_metadata(self):
        scenario = volumes.ModifyVolumeMetadata(self._get_context())
        scenario._set_metadata = mock.Mock()
        scenario._delete_metadata = mock.Mock()

        scenario.run(sets=5, set_size=4, deletes=3, delete_size=2)
        scenario._set_metadata.assert_called_once_with("uuid", 5, 4)
        scenario._delete_metadata.assert_called_once_with(
            "uuid",
            scenario._set_metadata.return_value, 3, 2)

    def test_create_and_extend_volume(self):
        fake_volume = mock.MagicMock()

        scenario = volumes.CreateAndExtendVolume(self.context)
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._extend_volume = mock.MagicMock(return_value=fake_volume)
        scenario.sleep_between = mock.MagicMock()
        scenario._delete_volume = mock.MagicMock()

        scenario.run(1, 2, 10, 20, fakearg="f")
        scenario._create_volume.assert_called_once_with(1, fakearg="f")
        self.assertTrue(scenario._extend_volume.called)
        scenario.sleep_between.assert_called_once_with(10, 20)
        scenario._delete_volume.assert_called_once_with(fake_volume)

    def test_create_from_image_and_delete_volume(self):
        fake_volume = mock.MagicMock()
        scenario = volumes.CreateAndDeleteVolume(self.context)
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_volume = mock.MagicMock()

        scenario.run(1, image="fake_image")
        scenario._create_volume.assert_called_once_with(1,
                                                        imageRef="fake_image")

        scenario._delete_volume.assert_called_once_with(fake_volume)

    def test_create_volume_from_image(self):
        fake_volume = mock.MagicMock()
        scenario = volumes.CreateVolume(self.context)
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)

        scenario.run(1, image="fake_image")
        scenario._create_volume.assert_called_once_with(1,
                                                        imageRef="fake_image")

    def test_create_volume_from_image_and_list(self):
        fake_volume = mock.MagicMock()
        scenario = volumes.CreateAndListVolume(self.context)
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._list_volumes = mock.MagicMock()

        scenario.run(1, True, "fake_image")
        scenario._create_volume.assert_called_once_with(1,
                                                        imageRef="fake_image")
        scenario._list_volumes.assert_called_once_with(True)

    def test_create_from_volume_and_delete_volume(self):
        fake_volume = mock.MagicMock()
        vol_size = 1
        scenario = volumes.CreateFromVolumeAndDeleteVolume(self._get_context())
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_volume = mock.MagicMock()

        scenario.run(vol_size)
        scenario._create_volume.assert_called_once_with(1, source_volid="uuid")
        scenario._delete_volume.assert_called_once_with(fake_volume)

    def test_create_and_delete_snapshot(self):
        fake_snapshot = mock.MagicMock()
        scenario = volumes.CreateAndDeleteSnapshot(self._get_context())

        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
        scenario.sleep_between = mock.MagicMock()
        scenario._delete_snapshot = mock.MagicMock()

        scenario.run(False, 10, 20, fakearg="f")

        scenario._create_snapshot.assert_called_once_with("uuid", force=False,
                                                          fakearg="f")
        scenario.sleep_between.assert_called_once_with(10, 20)
        scenario._delete_snapshot.assert_called_once_with(fake_snapshot)

    def test_create_and_list_snapshots(self):
        fake_snapshot = mock.MagicMock()
        scenario = volumes.CreateAndListSnapshots(self._get_context())

        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
        scenario._list_snapshots = mock.MagicMock()
        scenario.run(False, True, fakearg="f")
        scenario._create_snapshot.assert_called_once_with("uuid", force=False,
                                                          fakearg="f")
        scenario._list_snapshots.assert_called_once_with(True)

    def test_create_and_attach_volume(self):
        fake_volume = mock.MagicMock()
        fake_server = mock.MagicMock()
        fake_attach = mock.MagicMock()
        scenario = volumes.CreateAndAttachVolume(self.context)

        scenario._attach_volume = mock.MagicMock(return_value=fake_attach)
        scenario._detach_volume = mock.MagicMock()

        scenario._boot_server = mock.MagicMock(return_value=fake_server)
        scenario._delete_server = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_volume = mock.MagicMock()

        volume_args = {"some_key": "some_val"}
        vm_args = {"some_key": "some_val"}

        scenario.run(10, "img", "0",
                     create_volume_params=volume_args,
                     create_vm_params=vm_args)
        scenario._attach_volume.assert_called_once_with(fake_server,
                                                        fake_volume)
        scenario._detach_volume.assert_called_once_with(fake_server,
                                                        fake_volume,
                                                        fake_attach)

        scenario._delete_volume.assert_called_once_with(fake_volume)
        scenario._delete_server.assert_called_once_with(fake_server)

    def test_create_and_upload_volume_to_image(self):
        fake_volume = mock.Mock()
        fake_image = mock.Mock()
        scenario = volumes.CreateAndUploadVolumeToImage(self.context)

        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._upload_volume_to_image = mock.MagicMock(
            return_value=fake_image)
        scenario._delete_volume = mock.MagicMock()
        scenario._delete_image = mock.MagicMock()

        scenario.run(2, image="img", container_format="fake",
                     disk_format="disk", do_delete=False, fakeargs="fakeargs")

        scenario._create_volume.assert_called_once_with(2, imageRef="img",
                                                        fakeargs="fakeargs")
        scenario._upload_volume_to_image.assert_called_once_with(fake_volume,
                                                                 False,
                                                                 "fake",
                                                                 "disk")
        scenario._create_volume.reset_mock()
        scenario._upload_volume_to_image.reset_mock()

        scenario.run(1, image=None, do_delete=True, fakeargs="fakeargs")

        scenario._create_volume.assert_called_once_with(1, fakeargs="fakeargs")
        scenario._upload_volume_to_image.assert_called_once_with(fake_volume,
                                                                 False,
                                                                 "bare",
                                                                 "raw")
        scenario._delete_volume.assert_called_once_with(fake_volume)
        scenario._delete_image.assert_called_once_with(fake_image)

    def test_create_snapshot_and_attach_volume(self):
        fake_volume = mock.MagicMock()
        fake_snapshot = mock.MagicMock()
        fake_server = mock.MagicMock()
        fake_attach = mock.MagicMock()
        scenario = volumes.CreateSnapshotAndAttachVolume(self._get_context())

        scenario._attach_volume = mock.MagicMock(return_value=fake_attach)
        scenario._detach_volume = mock.MagicMock()
        scenario._boot_server = mock.MagicMock(return_value=fake_server)
        scenario._delete_server = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_volume = mock.MagicMock()
        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
        scenario._delete_snapshot = mock.MagicMock()

        self.clients("nova").servers.get = mock.MagicMock(
            return_value=fake_server)

        scenario.run()

        self.assertTrue(scenario._create_volume.called)
        scenario._create_snapshot.assert_called_once_with(fake_volume.id,
                                                          False)
        scenario._delete_snapshot.assert_called_once_with(fake_snapshot)
        scenario._attach_volume.assert_called_once_with(fake_server,
                                                        fake_volume)
        scenario._detach_volume.assert_called_once_with(fake_server,
                                                        fake_volume,
                                                        fake_attach)
        scenario._delete_volume.assert_called_once_with(fake_volume)

    def test_create_snapshot_and_attach_volume_use_volume_type_with_name(self):
        fake_volume = mock.MagicMock()
        fake_snapshot = mock.MagicMock()
        fake_server = mock.MagicMock()
        fake_attach = mock.MagicMock()

        scenario = volumes.CreateSnapshotAndAttachVolume(self._get_context())

        scenario._attach_volume = mock.MagicMock(return_value=fake_attach)
        scenario._detach_volume = mock.MagicMock()
        scenario._boot_server = mock.MagicMock(return_value=fake_server)
        scenario._delete_server = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_volume = mock.MagicMock()
        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
        scenario._delete_snapshot = mock.MagicMock()
        fake = fake_type()

        self.clients("cinder").volume_types.list = mock.MagicMock(
            return_value=[fake])
        self.clients("nova").servers.get = mock.MagicMock(
            return_value=fake_server)

        scenario.run(volume_type="fake_volume_type")

        # Make sure create volume's second arg was the correct volume type.
        # fake or none (randomly selected)
        self.assertTrue(scenario._create_volume.called)
        vol_type = scenario._create_volume.call_args_list[0][1]["volume_type"]
        self.assertEqual(vol_type, "fake_volume_type")
        scenario._create_snapshot.assert_called_once_with(fake_volume.id,
                                                          False)
        scenario._delete_snapshot.assert_called_once_with(fake_snapshot)
        scenario._attach_volume.assert_called_once_with(fake_server,
                                                        fake_volume)
        scenario._detach_volume.assert_called_once_with(fake_server,
                                                        fake_volume,
                                                        fake_attach)
        scenario._delete_volume.assert_called_once_with(fake_volume)

    def test_create_nested_snapshots_and_attach_volume(self):
        fake_volume = mock.MagicMock()
        fake_snapshot = mock.MagicMock()
        fake_attach = mock.MagicMock()

        scenario = volumes.CreateNestedSnapshotsAndAttachVolume(
            context=self._get_context())

        scenario._attach_volume = mock.MagicMock(return_value=fake_attach)
        scenario._detach_volume = mock.MagicMock()
        scenario._delete_server = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_volume = mock.MagicMock()
        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
        scenario._delete_snapshot = mock.MagicMock()

        scenario.run()

        volume_count = scenario._create_volume.call_count
        snapshots_count = scenario._create_snapshot.call_count
        attached_count = scenario._attach_volume.call_count

        self.assertEqual(scenario._delete_volume.call_count, volume_count)
        self.assertEqual(scenario._delete_snapshot.call_count, snapshots_count)
        self.assertEqual(scenario._detach_volume.call_count, attached_count)

    def test_create_nested_snapshots_and_attach_volume_kwargs(self):
        fake_volume = mock.MagicMock()
        fake_snapshot = mock.MagicMock()
        fake_attach = mock.MagicMock()

        scenario = volumes.CreateNestedSnapshotsAndAttachVolume(
            context=self._get_context())

        scenario._attach_volume = mock.MagicMock(return_value=fake_attach)
        scenario._detach_volume = mock.MagicMock()
        scenario._delete_server = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_volume = mock.MagicMock()
        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
        scenario._delete_snapshot = mock.MagicMock()

        volume_kwargs = {"volume_type": "type1"}
        scenario.run(size={"min": 1, "max": 1},
                     create_volume_kwargs=volume_kwargs)

        scenario._create_volume.assert_called_once_with(1, **volume_kwargs)
        self.assertEqual(fake_volume, scenario._create_volume.return_value)

    def test_create_nested_snapshots_and_attach_volume_snapshot_kwargs(self):
        fake_volume = mock.MagicMock()
        fake_volume.id = "FAKE_ID"
        fake_snapshot = mock.MagicMock()
        fake_attach = mock.MagicMock()

        scenario = volumes.CreateNestedSnapshotsAndAttachVolume(
            context=self._get_context())

        scenario._attach_volume = mock.MagicMock(return_value=fake_attach)
        scenario._detach_volume = mock.MagicMock()
        scenario._delete_server = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_volume = mock.MagicMock()
        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
        scenario._delete_snapshot = mock.MagicMock()

        volume_kwargs = {"volume_type": "type1"}
        snapshot_kwargs = {"name": "snapshot1", "description": "snaphot one"}
        scenario.run(size={"min": 1, "max": 1},
                     create_volume_kwargs=volume_kwargs,
                     create_snapshot_kwargs=snapshot_kwargs)

        scenario._create_snapshot.assert_called_once_with(fake_volume.id,
                                                          False,
                                                          **snapshot_kwargs)
        self.assertEqual(fake_snapshot, scenario._create_snapshot.return_value)

    def test_create_nested_snapshots_and_attach_volume_deprecate_kwargs(self):
        fake_volume = mock.MagicMock()
        fake_volume.id = "FAKE_ID"
        fake_snapshot = mock.MagicMock()
        fake_attach = mock.MagicMock()

        scenario = volumes.CreateNestedSnapshotsAndAttachVolume(
            self._get_context())

        scenario._attach_volume = mock.MagicMock(return_value=fake_attach)
        scenario._detach_volume = mock.MagicMock()
        scenario._delete_server = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_volume = mock.MagicMock()
        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
        scenario._delete_snapshot = mock.MagicMock()

        volume_kwargs = {"volume_type": "type1"}
        snapshot_kwargs = {"name": "snapshot1", "description": "snaphot one"}
        scenario.run(size={"min": 1, "max": 1},
                     create_volume_kwargs=volume_kwargs,
                     **snapshot_kwargs)

        scenario._create_snapshot.assert_called_once_with(fake_volume.id,
                                                          False,
                                                          **snapshot_kwargs)
        self.assertEqual(fake_snapshot, scenario._create_snapshot.return_value)

    def test_create_nested_snapshots_calls_order(self):
        fake_volume1 = mock.MagicMock()
        fake_volume2 = mock.MagicMock()
        fake_snapshot1 = mock.MagicMock()
        fake_snapshot2 = mock.MagicMock()

        scenario = volumes.CreateNestedSnapshotsAndAttachVolume(
            self._get_context())

        scenario._attach_volume = mock.MagicMock(return_value=mock.MagicMock())
        scenario._detach_volume = mock.MagicMock()
        scenario._delete_server = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(
            side_effect=[fake_volume1, fake_volume2])
        scenario._delete_volume = mock.MagicMock()
        scenario._create_snapshot = mock.MagicMock(
            side_effect=[fake_snapshot1, fake_snapshot2])
        scenario._delete_snapshot = mock.MagicMock()

        scenario.run(nested_level=2)

        vol_delete_calls = [mock.call(fake_volume2), mock.call(fake_volume1)]
        snap_delete_calls = [mock.call(fake_snapshot2),
                             mock.call(fake_snapshot1)]

        scenario._delete_volume.assert_has_calls(vol_delete_calls)
        scenario._delete_snapshot.assert_has_calls(snap_delete_calls)

    @mock.patch("rally.plugins.openstack.scenarios.cinder.volumes.random")
    def test_create_nested_snapshots_check_resources_size(self, mock_random):
        mock_random.randint.return_value = 3
        fake_volume = mock.MagicMock()
        fake_snapshot = mock.MagicMock()
        fake_server = mock.MagicMock()

        scenario = volumes.CreateNestedSnapshotsAndAttachVolume(
            self._get_context())

        scenario.get_random_server = mock.MagicMock(return_value=fake_server)
        scenario._attach_volume = mock.MagicMock(return_value=mock.MagicMock())
        scenario._detach_volume = mock.MagicMock()
        scenario._delete_server = mock.MagicMock()
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_volume = mock.MagicMock()
        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
        scenario._delete_snapshot = mock.MagicMock()

        scenario.run(nested_level=2)

        # NOTE: One call for random size
        random_call_count = mock_random.randint.call_count
        self.assertEqual(1, random_call_count)

        calls = scenario._create_volume.mock_calls
        expected_calls = [mock.call(3)]
        expected_calls.extend(
            [mock.call(3, snapshot_id=fake_snapshot.id)])
        self.assertEqual(expected_calls, calls)

    def test_create_volume_backup(self):
        fake_volume = mock.MagicMock()
        fake_backup = mock.MagicMock()
        scenario = volumes.CreateVolumeBackup(self.context)
        self._get_scenario(scenario, fake_volume, fake_backup)

        volume_kwargs = {"some_var": "zaq"}
        scenario.run(1, do_delete=True, create_volume_kwargs=volume_kwargs)
        scenario._create_volume.assert_called_once_with(1, **volume_kwargs)
        scenario._create_backup.assert_called_once_with(fake_volume.id)
        scenario._delete_volume.assert_called_once_with(fake_volume)
        scenario._delete_backup.assert_called_once_with(fake_backup)

    def test_create_volume_backup_no_delete(self):
        fake_volume = mock.MagicMock()
        fake_backup = mock.MagicMock()
        scenario = volumes.CreateVolumeBackup(self.context)
        self._get_scenario(scenario, fake_volume, fake_backup)

        volume_kwargs = {"some_var": "zaq"}
        scenario.run(1, do_delete=False, create_volume_kwargs=volume_kwargs)
        scenario._create_volume.assert_called_once_with(1, **volume_kwargs)
        scenario._create_backup.assert_called_once_with(fake_volume.id)
        self.assertFalse(scenario._delete_volume.called)
        self.assertFalse(scenario._delete_backup.called)

    def _get_scenario(self, scenario, fake_volume,
                      fake_backup, fake_restore=None):
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._create_backup = mock.MagicMock(return_value=fake_backup)
        scenario._restore_backup = mock.MagicMock(return_value=fake_restore)
        scenario._list_backups = mock.MagicMock()
        scenario._delete_volume = mock.MagicMock()
        scenario._delete_backup = mock.MagicMock()

    def test_create_and_restore_volume_backup(self):
        fake_volume = mock.MagicMock()
        fake_backup = mock.MagicMock()
        fake_restore = mock.MagicMock()
        scenario = volumes.CreateAndRestoreVolumeBackup(self.context)
        self._get_scenario(scenario, fake_volume, fake_backup, fake_restore)

        volume_kwargs = {"some_var": "zaq"}
        scenario.run(1, do_delete=True, create_volume_kwargs=volume_kwargs)
        scenario._create_volume.assert_called_once_with(1, **volume_kwargs)
        scenario._create_backup.assert_called_once_with(fake_volume.id)
        scenario._restore_backup.assert_called_once_with(fake_backup.id)
        scenario._delete_volume.assert_called_once_with(fake_volume)
        scenario._delete_backup.assert_called_once_with(fake_backup)

    def test_create_and_restore_volume_backup_no_delete(self):
        fake_volume = mock.MagicMock()
        fake_backup = mock.MagicMock()
        fake_restore = mock.MagicMock()
        scenario = volumes.CreateAndRestoreVolumeBackup(self.context)
        self._get_scenario(scenario, fake_volume, fake_backup, fake_restore)

        volume_kwargs = {"some_var": "zaq"}
        scenario.run(1, do_delete=False, create_volume_kwargs=volume_kwargs)
        scenario._create_volume.assert_called_once_with(1, **volume_kwargs)
        scenario._create_backup.assert_called_once_with(fake_volume.id)
        scenario._restore_backup.assert_called_once_with(fake_backup.id)
        self.assertFalse(scenario._delete_volume.called)
        self.assertFalse(scenario._delete_backup.called)

    def test_create_and_list_volume_backups(self):
        fake_volume = mock.MagicMock()
        fake_backup = mock.MagicMock()
        scenario = volumes.CreateAndListVolumeBackups(self.context)
        self._get_scenario(scenario, fake_volume, fake_backup)

        volume_kwargs = {"some_var": "zaq"}
        scenario.run(1, detailed=True, do_delete=True,
                     create_volume_kwargs=volume_kwargs)
        scenario._create_volume.assert_called_once_with(1, **volume_kwargs)
        scenario._create_backup.assert_called_once_with(fake_volume.id)
        scenario._list_backups.assert_called_once_with(True)
        scenario._delete_volume.assert_called_once_with(fake_volume)
        scenario._delete_backup.assert_called_once_with(fake_backup)

    def test_create_and_list_volume_backups_no_delete(self):
        fake_volume = mock.MagicMock()
        fake_backup = mock.MagicMock()
        scenario = volumes.CreateAndListVolumeBackups(self.context)
        self._get_scenario(scenario, fake_volume, fake_backup)

        volume_kwargs = {"some_var": "zaq"}
        scenario.run(1, detailed=True, do_delete=False,
                     create_volume_kwargs=volume_kwargs)
        scenario._create_volume.assert_called_once_with(1, **volume_kwargs)
        scenario._create_backup.assert_called_once_with(fake_volume.id)
        scenario._list_backups.assert_called_once_with(True)
        self.assertFalse(scenario._delete_volume.called)
        self.assertFalse(scenario._delete_backup.called)

    @ddt.data({},
              {"nested_level": 2},
              {"image": "img"})
    @ddt.unpack
    def test_create_volume_and_clone(self, nested_level=1,
                                     image=None):
        create_volumes_count = nested_level + 1
        fake_volumes = [mock.Mock(size=1) for i in range(create_volumes_count)]
        scenario = volumes.CreateVolumeAndClone(self.context)
        scenario._create_volume = mock.MagicMock(side_effect=fake_volumes)

        scenario.run(1, image=image, nested_level=nested_level, fakearg="fake")

        expected = [mock.call(1, imageRef=image, fakearg="fake")
                    if image else mock.call(1, fakearg="fake")]
        for i in range(nested_level):
            expected.append(mock.call(fake_volumes[i].size,
                                      source_volid=fake_volumes[i].id,
                                      atomic_action=False, fakearg="fake")
                            )
            self._test_atomic_action_timer(scenario.atomic_actions(),
                                           "cinder.clone_volume")
        scenario._create_volume.assert_has_calls(expected)

    def test_create_volume_from_snapshot(self):
        fake_snapshot = mock.MagicMock(id=1)
        fake_volume = mock.MagicMock()
        create_snapshot_args = {"force": False}
        scenario = volumes.CreateVolumeFromSnapshot(self._get_context())

        scenario._create_snapshot = mock.MagicMock(return_value=fake_snapshot)
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._delete_snapshot = mock.MagicMock()
        scenario._delete_volume = mock.MagicMock()

        scenario.run(fakearg="f")

        scenario._create_snapshot.assert_called_once_with("uuid")
        scenario._create_volume.assert_called_once_with(
            1, snapshot_id=fake_snapshot.id, fakearg="f")
        scenario._delete_snapshot.assert_called_once_with(fake_snapshot)
        scenario._delete_volume.assert_called_once_with(fake_volume)

        scenario._create_snapshot.reset_mock()
        scenario._create_volume.reset_mock()
        scenario._delete_snapshot.reset_mock()
        scenario._delete_volume.reset_mock()

        scenario.run(do_delete=False,
                     create_snapshot_kwargs=create_snapshot_args,
                     fakearg="f")

        scenario._create_snapshot.assert_called_once_with(
            "uuid", **create_snapshot_args)
        scenario._create_volume.assert_called_once_with(
            1, snapshot_id=fake_snapshot.id, fakearg="f")
        self.assertFalse(scenario._delete_snapshot.called)
        self.assertFalse(scenario._delete_volume.called)

    @ddt.data({},
              {"image": "img"})
    @ddt.unpack
    def test_create_and_accept_transfer(self, image=None):
        fake_volume = mock.MagicMock()
        fake_transfer = mock.MagicMock()
        scenario = volumes.CreateAndAcceptTransfer(self._get_context())
        scenario._create_volume = mock.MagicMock(return_value=fake_volume)
        scenario._transfer_create = mock.MagicMock(return_value=fake_transfer)
        scenario._transfer_accept = mock.MagicMock()
        scenario.run(1, image=image, fakearg="fake")
        expected = [mock.call(1, imageRef=image, fakearg="fake")
                    if image else mock.call(1, fakearg="fake")]
        scenario._create_volume.assert_has_calls(expected)
        scenario._transfer_create.assert_called_once_with(fake_volume.id)
        scenario._transfer_accept.assert_called_once_with(
            fake_transfer.id, fake_transfer.auth_key)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import tempfile

import ddt
from glanceclient import exc as glance_exc
import mock
from oslo_config import cfg

from rally import exceptions
from rally.plugins.openstack.wrappers import glance as glance_wrapper
from tests.unit import test

CONF = cfg.CONF


@ddt.ddt
class GlanceWrapperTestCase(test.ScenarioTestCase):

    @ddt.data(
        {"version": "1", "expected_class": glance_wrapper.GlanceV1Wrapper},
        {"version": "2", "expected_class": glance_wrapper.GlanceV2Wrapper}
    )
    @ddt.unpack
    def test_wrap(self, version, expected_class):
        client = mock.MagicMock()
        client.choose_version.return_value = version
        self.assertIsInstance(glance_wrapper.wrap(client, mock.Mock()),
                              expected_class)

    @mock.patch("rally.plugins.openstack.wrappers.glance.LOG")
    def test_wrap_wrong_version(self, mock_log):
        client = mock.MagicMock()
        client.choose_version.return_value = "dummy"
        self.assertRaises(exceptions.InvalidArgumentsException,
                          glance_wrapper.wrap, client, mock.Mock())
        self.assertTrue(mock_log.warning.mock_called)


@ddt.ddt
class GlanceV1WrapperTestCase(test.ScenarioTestCase):
    _tempfile = tempfile.NamedTemporaryFile()

    def setUp(self):
        super(GlanceV1WrapperTestCase, self).setUp()
        self.client = mock.MagicMock()
        self.client.choose_version.return_value = "1"
        self.owner = mock.Mock()
        self.wrapped_client = glance_wrapper.wrap(self.client, self.owner)

    def test_get_image(self):
        image = mock.Mock()

        return_image = self.wrapped_client.get_image(image)

        self.client.return_value.images.get.assert_called_once_with(image.id)
        self.assertEqual(return_image,
                         self.client.return_value.images.get.return_value)

    def test_get_image_not_found(self):
        image = mock.Mock()
        self.client.return_value.images.get.side_effect = (
            glance_exc.HTTPNotFound)

        self.assertRaises(exceptions.GetResourceNotFound,
                          self.wrapped_client.get_image, image)
        self.client.return_value.images.get.assert_called_once_with(image.id)

    @ddt.data(
        {"location": "image_location", "visibility": "private"},
        {"location": "image_location", "fakearg": "fake"},
        {"location": "image_location", "name": "image_name"},
        {"location": _tempfile.name, "visibility": "public"})
    @ddt.unpack
    @mock.patch("six.moves.builtins.open")
    def test_create_image(self, mock_open, location, **kwargs):
        return_image = self.wrapped_client.create_image("container_format",
                                                        location,
                                                        "disk_format",
                                                        **kwargs)
        call_args = kwargs
        call_args["container_format"] = "container_format"
        call_args["disk_format"] = "disk_format"
        if location.startswith("/"):
            call_args["data"] = mock_open.return_value
            mock_open.assert_called_once_with(location)
            mock_open.return_value.close.assert_called_once_with()
        else:
            call_args["copy_from"] = location
        if "name" not in kwargs:
            call_args["name"] = self.owner.generate_random_name.return_value
        if "visibility" in kwargs:
            call_args["is_public"] = call_args.pop("visibility") == "public"

        self.client().images.create.assert_called_once_with(**call_args)

        self.mock_wait_for_status.mock.assert_called_once_with(
            self.client().images.create.return_value, ["active"],
            update_resource=self.wrapped_client.get_image,
            check_interval=CONF.benchmark.glance_image_create_poll_interval,
            timeout=CONF.benchmark.glance_image_create_timeout)
        self.assertEqual(self.mock_wait_for_status.mock.return_value,
                         return_image)

    @ddt.data({"expected": True},
              {"visibility": "public", "expected": True},
              {"visibility": "private", "expected": False})
    @ddt.unpack
    def test_set_visibility(self, visibility=None, expected=None):
        image = mock.Mock()
        if visibility is None:
            self.wrapped_client.set_visibility(image)
        else:
            self.wrapped_client.set_visibility(image, visibility=visibility)
        self.client().images.update.assert_called_once_with(
            image.id, is_public=expected)

    @ddt.data({}, {"fakearg": "fake"})
    def test_list_images_basic(self, filters):
        self.assertEqual(self.wrapped_client.list_images(**filters),
                         self.client().images.list.return_value)
        self.client().images.list.assert_called_once_with(filters=filters)

    def test_list_images_with_owner(self):
        self.assertEqual(self.wrapped_client.list_images(fakearg="fake",
                                                         owner="fakeowner"),
                         self.client().images.list.return_value)
        self.client().images.list.assert_called_once_with(
            owner="fakeowner", filters={"fakearg": "fake"})

    def test_list_images_visibility_public(self):
        public_images = [mock.Mock(is_public=True), mock.Mock(is_public=True)]
        private_images = [mock.Mock(is_public=False),
                          mock.Mock(is_public=False)]
        self.client().images.list.return_value = public_images + private_images
        self.assertEqual(self.wrapped_client.list_images(fakearg="fake",
                                                         visibility="public"),
                         public_images)
        self.client().images.list.assert_called_once_with(
            filters={"fakearg": "fake"})

    def test_list_images_visibility_private(self):
        public_images = [mock.Mock(is_public=True), mock.Mock(is_public=True)]
        private_images = [mock.Mock(is_public=False),
                          mock.Mock(is_public=False)]
        self.client().images.list.return_value = public_images + private_images
        self.assertEqual(self.wrapped_client.list_images(fakearg="fake",
                                                         visibility="private"),
                         private_images)
        self.client().images.list.assert_called_once_with(
            filters={"fakearg": "fake"})


@ddt.ddt
class GlanceV2WrapperTestCase(test.ScenarioTestCase):
    _tempfile = tempfile.NamedTemporaryFile()

    def setUp(self):
        super(GlanceV2WrapperTestCase, self).setUp()
        self.client = mock.MagicMock()
        self.client.choose_version.return_value = "2"
        self.owner = mock.Mock()
        self.wrapped_client = glance_wrapper.wrap(self.client, self.owner)

    def test_get_image(self):
        image = mock.Mock()

        return_image = self.wrapped_client.get_image(image)

        self.client.return_value.images.get.assert_called_once_with(image.id)
        self.assertEqual(return_image,
                         self.client.return_value.images.get.return_value)

    def test_get_image_not_found(self):
        image = mock.Mock()
        self.client.return_value.images.get.side_effect = (
            glance_exc.HTTPNotFound)

        self.assertRaises(exceptions.GetResourceNotFound,
                          self.wrapped_client.get_image, image)
        self.client.return_value.images.get.assert_called_once_with(image.id)

    @ddt.data(
        {"location": "image_location", "visibility": "private"},
        {"location": "image_location", "fakearg": "fake"},
        {"location": "image_location", "name": "image_name"},
        {"location": _tempfile.name, "visibility": "public"},
        {"location": "image_location",
         "expected_kwargs": {"visibility": "public"}, "is_public": True})
    @ddt.unpack
    @mock.patch("six.moves.builtins.open")
    @mock.patch("requests.get")
    def test_create_image(self, mock_requests_get, mock_open, location,
                          expected_kwargs=None, **kwargs):
        self.wrapped_client.get_image = mock.Mock()
        created_image = mock.Mock()
        uploaded_image = mock.Mock()
        self.mock_wait_for_status.mock.side_effect = [created_image,
                                                      uploaded_image]

        return_image = self.wrapped_client.create_image("container_format",
                                                        location,
                                                        "disk_format",
                                                        **kwargs)
        create_args = expected_kwargs or kwargs
        create_args["container_format"] = "container_format"
        create_args["disk_format"] = "disk_format"
        create_args.setdefault("name",
                               self.owner.generate_random_name.return_value)

        self.client().images.create.assert_called_once_with(**create_args)

        if location.startswith("/"):
            data = mock_open.return_value
            mock_open.assert_called_once_with(location)
        else:
            data = mock_requests_get.return_value.raw
            mock_requests_get.assert_called_once_with(location, stream=True)
        data.close.assert_called_once_with()
        self.client().images.upload.assert_called_once_with(created_image.id,
                                                            data)

        self.mock_wait_for_status.mock.assert_has_calls([
            mock.call(
                self.client().images.create.return_value, ["queued"],
                update_resource=self.wrapped_client.get_image,
                check_interval=CONF.benchmark.
                glance_image_create_poll_interval,
                timeout=CONF.benchmark.glance_image_create_timeout),
            mock.call(
                created_image, ["active"],
                update_resource=self.wrapped_client.get_image,
                check_interval=CONF.benchmark.
                glance_image_create_poll_interval,
                timeout=mock.ANY)])
        self.assertEqual(uploaded_image, return_image)

    @ddt.data({},
              {"visibility": "public"},
              {"visibility": "private"})
    @ddt.unpack
    def test_set_visibility(self, visibility=None):
        image = mock.Mock()
        if visibility is None:
            self.wrapped_client.set_visibility(image)
            visibility = "public"
        else:
            self.wrapped_client.set_visibility(image, visibility=visibility)
        self.client().images.update.assert_called_once_with(
            image.id, visibility=visibility)

    @ddt.data({}, {"fakearg": "fake"})
    def test_list_images(self, filters):
        self.assertEqual(self.wrapped_client.list_images(**filters),
                         self.client().images.list.return_value)
        self.client().images.list.assert_called_once_with(filters=filters)



# Copyright 2015: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from rally import consts
from rally.plugins.openstack import scenario
from rally.plugins.openstack.scenarios.nova import utils
from rally.task import validation


"""Scenarios for Nova networks."""


@validation.restricted_parameters("label")
@validation.required_parameters("start_cidr")
@validation.required_services(consts.Service.NOVA, consts.Service.NOVA_NET)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["nova.networks"]},
                    name="NovaNetworks.create_and_list_networks")
class CreateAndListNetworks(utils.NovaScenario):

    def run(self, start_cidr, **kwargs):
        """Create nova network and list all networks.

        :param start_cidr: IP range
        :param kwargs: Optional additional arguments for network creation
        """

        network = self._create_network(start_cidr, **kwargs)
        msg = ("Network isn't created")
        self.assertTrue(network, err_msg=msg)
        list_networks = self._list_networks()
        msg = ("New network not in the list of existed networks.\n"
               "New network UUID: {}\n"
               "List of available networks: {}").format(network,
                                                        list_networks)
        self.assertIn(network, list_networks, err_msg=msg)


@validation.restricted_parameters("label")
@validation.required_parameters("start_cidr")
@validation.required_services(consts.Service.NOVA, consts.Service.NOVA_NET)
@validation.required_openstack(admin=True)
@scenario.configure(context={"admin_cleanup": ["nova.networks"]},
                    name="NovaNetworks.create_and_delete_network")
class CreateAndDeleteNetwork(utils.NovaScenario):

    def run(self, start_cidr, **kwargs):
        """Create nova network and delete it.

        :param start_cidr: IP range
        :param kwargs: Optional additional arguments for network creation
        """

        net_id = self._create_network(start_cidr, **kwargs)
        self._delete_network(net_id)
